@@ -1,12 +1,12 @@
-/*  $Id: archive.c,v 1.19.2.1 2000/07/23 04:11:41 kondou Exp $
+/*  $Id: archive.c,v 1.38 2003/01/19 04:13:44 rra Exp $
 **
 **  Read batchfiles on standard input and archive them.
 */
+
 #include "config.h"
 #include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
-#include <syslog.h> 
 #include <sys/stat.h>
 #include <time.h>
 
@@ -14,44 +14,45 @@
 # include <sys/time.h>
 #endif
 
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/wire.h"
 #include "libinn.h"
-#include "macros.h"
 #include "paths.h"
-#include "qio.h"
+#include "storage.h"
 
 
-STATIC char	*Archive = NULL;
-STATIC char	*ERRLOG = NULL;
+static char	*Archive = NULL;
+static char	*ERRLOG = NULL;
 
 /*
 **  Return a YYYYMM string that represents the current year/month
 */
-STATIC char *
-DateString()
+static char *
+DateString(void)
 {
-    STATIC char		ds[10];
+    static char		ds[10];
     time_t		now;
     struct tm		*x;
 
     time(&now);
     x = localtime(&now);
-    sprintf(ds, "%d%d", x->tm_year + 1900, x->tm_mon + 1);
+    snprintf(ds, sizeof(ds), "%d%d", x->tm_year + 1900, x->tm_mon + 1);
 
     return ds;
 }
 
 
 /*
-**  Try to make one directory.  Return FALSE on error.
+**  Try to make one directory.  Return false on error.
 */
-STATIC BOOL
-MakeDir(Name)
-    char		*Name;
+static bool
+MakeDir(char *Name)
 {
     struct stat		Sb;
 
     if (mkdir(Name, GROUPDIR_MODE) >= 0)
-	return TRUE;
+	return true;
 
     /* See if it failed because it already exists. */
     return stat(Name, &Sb) >= 0 && S_ISDIR(Sb.st_mode);
@@ -60,15 +61,14 @@ MakeDir(Name)
 
 /*
 **  Given an entry, comp/foo/bar/1123, create the directory and all
-**  parent directories needed.  Return FALSE on error.
+**  parent directories needed.  Return false on error.
 */
-STATIC BOOL
-MakeArchiveDirectory(Name)
-    register char	*Name;
+static bool
+MakeArchiveDirectory(char *Name)
 {
-    register char	*p;
-    register char	*save;
-    BOOL		made;
+    char	*p;
+    char	*save;
+    bool		made;
 
     if ((save = strrchr(Name, '/')) != NULL)
 	*save = '\0';
@@ -77,7 +77,7 @@ MakeArchiveDirectory(Name)
     if (MakeDir(Name)) {
 	if (save)
 	    *save = '/';
-	return TRUE;
+	return true;
     }
 
     /* Try to make each of comp and comp/foo in turn. */
@@ -89,7 +89,7 @@ MakeArchiveDirectory(Name)
 	    if (!made) {
 		if (save)
 		    *save = '/';
-		return FALSE;
+		return false;
 	    }
 	}
 
@@ -101,88 +101,80 @@ MakeArchiveDirectory(Name)
 
 
 /*
-**  Copy a file.  Return FALSE if error.
+**  Copy a file.  Return false if error.
 */
-STATIC BOOL
-Copy(src, dest)
-    char		*src;
-    char		*dest;
+static bool
+Copy(char *src, char *dest)
 {
-    register FILE	*in;
-    register FILE	*out;
-    register SIZE_T	i;
-    char		*p;
-    char		buff[BUFSIZ];
+    FILE	*in;
+    FILE	*out;
+    size_t	i;
+    char	*p;
+    char	buff[BUFSIZ];
 
     /* Open the output file. */
     if ((out = fopen(dest, "w")) == NULL) {
 	/* Failed; make any missing directories and try again. */
 	if ((p = strrchr(dest, '/')) != NULL) {
 	    if (!MakeArchiveDirectory(dest)) {
-		(void)fprintf(stderr, "Can't mkdir for \"%s\", %s\n",
-			dest, strerror(errno));
-		return FALSE;
+                syswarn("cannot mkdir for %s", dest);
+		return false;
 	    }
 	    out = fopen(dest, "w");
 	}
 	if (p == NULL || out == NULL) {
-	    (void)fprintf(stderr, "Can't open \"%s\" for writing, %s\n",
-		    src, strerror(errno));
-	    return FALSE;
+            syswarn("cannot open %s for writing", dest);
+	    return false;
 	}
     }
 
     /* Opening the input file is easier. */
     if ((in = fopen(src, "r")) == NULL) {
-	(void)fprintf(stderr, "Can't open \"%s\" for reading, %s\n",
-		src, strerror(errno));
-	(void)fclose(out);
-	(void)unlink(dest);
-	return FALSE;
+        syswarn("cannot open %s for reading", src);
+	fclose(out);
+	unlink(dest);
+	return false;
     }
 
     /* Write the data. */
-    while ((i = fread((POINTER)buff, (SIZE_T)1, (SIZE_T)sizeof buff, in)) != 0)
-	if (fwrite((POINTER)buff, (SIZE_T)1, i, out) != i) {
-	    (void)fprintf(stderr, "Can't write \"%s\", %s\n",
-		    dest, strerror(errno));
-	    (void)fclose(in);
-	    (void)fclose(out);
-	    (void)unlink(dest);
-	    return FALSE;
+    while ((i = fread(buff, 1, sizeof buff, in)) != 0)
+	if (fwrite(buff, 1, i, out) != i) {
+            syswarn("cannot write to %s", dest);
+	    fclose(in);
+	    fclose(out);
+	    unlink(dest);
+	    return false;
 	}
-    (void)fclose(in);
+    fclose(in);
 
     /* Flush and close the output. */
     if (ferror(out) || fflush(out) == EOF) {
-	(void)fprintf(stderr, "Can't close \"%s\", %s\n",
-		dest, strerror(errno));
-	(void)unlink(dest);
-	(void)fclose(out);
-	return FALSE;
+        syswarn("cannot flush %s", dest);
+	unlink(dest);
+	fclose(out);
+	return false;
     }
     if (fclose(out) == EOF) {
-	(void)fprintf(stderr, "Can't close \"%s\", %s\n",
-		dest, strerror(errno));
-	(void)unlink(dest);
-	return FALSE;
+        syswarn("cannot close %s", dest);
+	unlink(dest);
+	return false;
     }
 
-    return TRUE;
+    return true;
 }
 
 
 /*
 **  Copy an article from memory into a file.
 */
-STATIC BOOL
-CopyArt(ARTHANDLE *art, char *dest, BOOL Concat)
+static bool
+CopyArt(ARTHANDLE *art, char *dest, bool Concat)
 {
-    register FILE	*out;
-    char		*p,*q,*last;
-    ARTHANDLE		article;
+    FILE	*out;
+    const char		*p;
+    char		*q, *article;
     size_t		i;
-    char		*mode = "w";
+    const char		*mode = "w";
 
     if (Concat) mode = "a";
 
@@ -191,22 +183,20 @@ CopyArt(ARTHANDLE *art, char *dest, BOOL Concat)
 	/* Failed; make any missing directories and try again. */
 	if ((p = strrchr(dest, '/')) != NULL) {
 	    if (!MakeArchiveDirectory(dest)) {
-		(void)fprintf(stderr, "Can't mkdir for \"%s\", %s\n",
-			dest, strerror(errno));
-		return FALSE;
+                syswarn("cannot mkdir for %s", dest);
+		return false;
 	    }
 	    out = fopen(dest, mode);
 	}
 	if (p == NULL || out == NULL) {
-	    (void)fprintf(stderr, "Can't open \"%s\" for writing, %s\n",
-		    dest, strerror(errno));
-	    return FALSE;
+            syswarn("cannot open %s for writing", dest);
+	    return false;
 	}
     }
 
     /* Copy the data. */
-    article.data = NEW(char, art->len);
-    for (i=0, last=NULL, q=article.data, p=art->data; p<art->data+art->len;) {
+    article = xmalloc(art->len);
+    for (i=0, q=article, p=art->data; p<art->data+art->len;) {
 	if (&p[1] < art->data + art->len && p[0] == '\r' && p[1] == '\n') {
 	    p += 2;
 	    *q++ = '\n';
@@ -231,51 +221,45 @@ CopyArt(ARTHANDLE *art, char *dest, BOOL Concat)
 	/* Write a separator... */
 	fprintf(out, "-----------\n");
     }
-    if (fwrite(article.data, i, 1, out) != 1) {
-	(void)fprintf(stderr, "Can't write \"%s\", %s\n",
-		dest, strerror(errno));
-	(void)fclose(out);
-	if (!Concat) (void)unlink(dest);
-	DISPOSE(article.data);
-	return FALSE;
+    if (fwrite(article, i, 1, out) != 1) {
+        syswarn("cannot write to %s", dest);
+	fclose(out);
+	if (!Concat) unlink(dest);
+	free(article);
+	return false;
     }
-    DISPOSE(article.data);
+    free(article);
 
     /* Flush and close the output. */
     if (ferror(out) || fflush(out) == EOF) {
-	(void)fprintf(stderr, "Can't close \"%s\", %s\n",
-		dest, strerror(errno));
-	if (!Concat) (void)unlink(dest);
-	(void)fclose(out);
-	return FALSE;
+        syswarn("cannot flush %s", dest);
+	if (!Concat) unlink(dest);
+	fclose(out);
+	return false;
     }
     if (fclose(out) == EOF) {
-	(void)fprintf(stderr, "Can't close \"%s\", %s\n",
-		dest, strerror(errno));
-	if (!Concat) (void)unlink(dest);
-	return FALSE;
+        syswarn("cannot close %s", dest);
+	if (!Concat) unlink(dest);
+	return false;
     }
 
-    return TRUE;
+    return true;
 }
 
 
 /*
 **  Write an index entry.  Ignore I/O errors; our caller checks for them.
 */
-STATIC void
-WriteArtIndex(art, FullName, ShortName)
-    ARTHANDLE		*art;
-    char		*FullName;
-    char		*ShortName;
+static void
+WriteArtIndex(ARTHANDLE *art, char *ShortName)
 {
-    register char	*p;
-    register int	i;
+    const char	*p;
+    int	i;
     char		Subject[BUFSIZ];
     char		MessageID[BUFSIZ];
 
     Subject[0] = '\0';		/* default to null string */
-    p = (char *)HeaderFindMem(art->data, art->len, "Subject", 7);
+    p = wire_findheader(art->data, art->len, "Subject");
     if (p != NULL) {
 	for (i=0; *p != '\r' && *p != '\n' && *p != '\0'; i++) {
 	    Subject[i] = *p++;
@@ -284,7 +268,7 @@ WriteArtIndex(art, FullName, ShortName)
     }
 
     MessageID[0] = '\0';	/* default to null string */
-    p = (char *)HeaderFindMem(art->data, art->len, "Message-ID", 10);
+    p = wire_findheader(art->data, art->len, "Message-ID");
     if (p != NULL) {
 	for (i=0; *p != '\r' && *p != '\n' && *p != '\0'; i++) {
 	    MessageID[i] = *p++;
@@ -292,24 +276,13 @@ WriteArtIndex(art, FullName, ShortName)
 	MessageID[i] = '\0';
     }
 
-    (void)printf("%s %s %s\n",
+    printf("%s %s %s\n",
 	    ShortName,
 	    MessageID[0] ? MessageID : "<none>",
 	    Subject[0] ? Subject : "<none>");
 }
 
 
-/*
-**  Print a usage message and exit.
-*/
-STATIC NORETURN
-Usage()
-{
-    (void)fprintf(stderr, "Usage error.\n");
-    exit(1);
-}
-
-
 /*
 ** Crack an Xref line apart into separate strings, each of the form "ng:artnum".
 ** Return in "lenp" the number of newsgroups found.
@@ -317,25 +290,24 @@ Usage()
 ** This routine blatantly stolen from tradspool.c
 */
 static char **
-CrackXref(char *xref, unsigned int *lenp) {
+CrackXref(const char *xref, unsigned int *lenp) {
     char *p;
     char **xrefs;
     char *q;
     unsigned int len, xrefsize;
-    unsigned int slen;
 
     len = 0;
     xrefsize = 5;
-    xrefs = NEW(char *, xrefsize);
+    xrefs = xmalloc(xrefsize * sizeof(char *));
 
     /* skip pathhost */
     if ((p = strchr(xref, ' ')) == NULL) {
-	fprintf(stderr, "archive: Could not find pathhost in Xref header");
+        warn("cannot find pathhost in Xref header");
 	return NULL;
     }
     /* skip next spaces */
     for (p++; *p == ' ' ; p++) ;
-    while (TRUE) {
+    while (true) {
 	/* check for EOL */
 	/* shouldn't ever hit null w/o hitting a \r\n first, but best to be paranoid */
 	if (*p == '\n' || *p == '\r' || *p == 0) {
@@ -346,15 +318,12 @@ CrackXref(char *xref, unsigned int *lenp) {
 	/* skip to next space or EOL */
 	for (q=p; *q && *q != ' ' && *q != '\n' && *q != '\r' ; ++q) ;
 
-	slen = q-p;
-	xrefs[len] = NEW(char, slen+1);
-	strncpy(xrefs[len], p, slen);
-	xrefs[len][slen] = '\0';
+        xrefs[len] = xstrndup(p, q - p);
 
 	if (++len == xrefsize) {
 	    /* grow xrefs if needed. */
 	    xrefsize *= 2;
-	    RENEW(xrefs, char *, xrefsize);
+            xrefs = xrealloc(xrefs, xrefsize * sizeof(char *));
 	}
 
  	p = q;
@@ -374,15 +343,14 @@ CrackGroups(char *group, unsigned int *lenp) {
     char **groups;
     char *q;
     unsigned int len, grpsize;
-    unsigned int slen;
 
     len = 0;
     grpsize = 5;
-    groups = NEW(char *, grpsize);
+    groups = xmalloc(grpsize * sizeof(char *));
 
     /* skip leading spaces */
     for (p=group; *p == ' ' ; p++) ;
-    while (TRUE) {
+    while (true) {
 	/* check for EOL */
 	/* shouldn't ever hit null w/o hitting a \r\n first, but best to be paranoid */
 	if (*p == '\n' || *p == '\r' || *p == 0) {
@@ -393,15 +361,12 @@ CrackGroups(char *group, unsigned int *lenp) {
 	/* skip to next comma, space, or EOL */
 	for (q=p; *q && *q != ',' && *q != ' ' && *q != '\n' && *q != '\r' ; ++q) ;
 
-	slen = q-p;
-	groups[len] = NEW(char, slen+1);
-	strncpy(groups[len], p, slen);
-	groups[len][slen] = '\0';
+        groups[len] = xstrndup(p, q - p);
 
 	if (++len == grpsize) {
 	    /* grow groups if needed. */
 	    grpsize *= 2;
-	    RENEW(groups, char *, grpsize);
+            groups = xrealloc(groups, grpsize * sizeof(char *));
 	}
 
  	p = q;
@@ -412,42 +377,41 @@ CrackGroups(char *group, unsigned int *lenp) {
 
 
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(int ac, char *av[])
 {
-    register char	*Name;
-    register char	*p;
-    register FILE	*F;
-    register int	i;
-    BOOL		Flat;
-    BOOL		Redirect;
-    BOOL		Concat;
+    char	*Name;
+    char	*p;
+    FILE	*F;
+    int	i;
+    bool		Flat;
+    bool		Redirect;
+    bool		Concat;
     char		*Index;
     char		buff[BUFSIZ];
-    char		temp[BUFSIZ];
+    char		*spool;
     char		dest[BUFSIZ];
     char		**groups, *q, *ng;
     char		**xrefs;
-    char		*xrefhdr;
+    const char		*xrefhdr;
     ARTHANDLE		*art;
     TOKEN		token;
     unsigned int	numgroups, numxrefs;
     int			j;
     char		*base = NULL;
-    BOOL		doit;
+    bool		doit;
 
-    /* First thing, set up logging and our identity. */
-    openlog("archive", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
+    /* First thing, set up our identity. */
+    message_program_name = "archive";
 
     /* Set defaults. */
-    if (ReadInnConf() < 0) exit(1);
-    Concat = FALSE;
-    Flat = FALSE;
+    if (!innconf_read(NULL))
+        exit(1);
+    Concat = false;
+    Flat = false;
     Index = NULL;
-    Redirect = TRUE;
-    (void)umask(NEWSUMASK);
-    ERRLOG = COPY(cpcatpath(innconf->pathlog, _PATH_ERRLOG));
+    Redirect = true;
+    umask(NEWSUMASK);
+    ERRLOG = concatpath(innconf->pathlog, _PATH_ERRLOG);
     Archive = innconf->patharchive;
     groups = NULL;
     numgroups = 0;
@@ -456,17 +420,17 @@ main(ac, av)
     while ((i = getopt(ac, av, "a:cfi:p:r")) != EOF)
 	switch (i) {
 	default:
-	    Usage();
-	    /* NOTREACHED */
+            die("usage error");
+            break;
 	case 'a':
 	    Archive = optarg;
 	    break;
 	case 'c':
-	    Flat = TRUE;
-	    Concat = TRUE;
+	    Flat = true;
+	    Concat = true;
 	    break;
 	case 'f':
-	    Flat = TRUE;
+	    Flat = true;
 	    break;
 	case 'i':
 	    Index = optarg;
@@ -475,7 +439,7 @@ main(ac, av)
 	    groups = CrackGroups(optarg, &numgroups);
 	    break;
 	case 'r':
-	    Redirect = FALSE;
+	    Redirect = false;
 	    break;
 	}
 
@@ -483,49 +447,36 @@ main(ac, av)
     ac -= optind;
     av += optind;
     if (ac > 2)
-	Usage();
+        die("usage error");
 
     /* Do file redirections. */
     if (Redirect)
-	(void)freopen(ERRLOG, "a", stderr);
-    if (ac == 1 && freopen(av[0], "r", stdin) == NULL) {
-	(void)fprintf(stderr, "archive:  Can't open \"%s\" for input, %s\n",
-		av[0], strerror(errno));
-	    exit(1);
-    }
-    if (Index && freopen(Index, "a", stdout) == NULL) {
-	(void)fprintf(stderr, "archive:  Can't open \"%s\" for output, %s\n",
-		Index, strerror(errno));
-	exit(1);
-    }
+	freopen(ERRLOG, "a", stderr);
+    if (ac == 1 && freopen(av[0], "r", stdin) == NULL)
+        sysdie("cannot open %s for input", av[0]);
+    if (Index && freopen(Index, "a", stdout) == NULL)
+        sysdie("cannot open %s for output", Index);
 
     /* Go to where the action is. */
-    if (chdir(innconf->patharticles) < 0) {
-	(void)fprintf(stderr, "archive:  Can't cd to \"%s\", %s\n",
-		innconf->patharticles, strerror(errno));
-	exit(1);
-    }
+    if (chdir(innconf->patharticles) < 0)
+        sysdie("cannot chdir to %s", innconf->patharticles);
 
     /* Set up the destination. */
-    (void)strcpy(dest, Archive);
+    strcpy(dest, Archive);
     Name = dest + strlen(dest);
     *Name++ = '/';
 
-    if (!SMinit()) {
-	(void)fprintf(stderr, "archive: Could not initialize the storage manager: %s", SMerrorstr);
-	exit(1);
-    }
+    if (!SMinit())
+        die("cannot initialize storage manager: %s", SMerrorstr);
 
     /* Read input. */
     while (fgets(buff, sizeof buff, stdin) != NULL) {
 	if ((p = strchr(buff, '\n')) == NULL) {
-	    (void)fprintf(stderr,
-		    "archive:  Skipping \"%.40s...\" -- too long\n",
-		    buff);
+            warn("skipping %.40s: too long", buff);
 	    continue;
 	}
 	*p = '\0';
-	if (buff[0] == '\0' || buff[0] == COMMENT_CHAR)
+	if (buff[0] == '\0' || buff[0] == '#')
 	    continue;
 
 	/* Check to see if this is a token... */
@@ -533,46 +484,45 @@ main(ac, av)
 	    /* Get a copy of the article. */
 	    token = TextToToken(buff);
 	    if ((art = SMretrieve(token, RETR_ALL)) == NULL) {
-		(void)fprintf(stderr, "Could not retrieve %s\n",
-			buff);
+                warn("cannot retrieve %s", buff);
 		continue;
 	    }
 
 	    /* Determine groups from the Xref header */
-    	    xrefhdr = (char *)HeaderFindMem(art->data, art->len, "Xref", 4);
+    	    xrefhdr = wire_findheader(art->data, art->len, "Xref");
 	    if (xrefhdr == NULL) {
-		fprintf(stderr, "archive: cant find Xref: header");
+                warn("cannot find Xref header");
 		SMfreearticle(art);
 		continue;
 	    }
 
 	    if ((xrefs = CrackXref(xrefhdr, &numxrefs)) == NULL || numxrefs == 0) {
-		fprintf(stderr, "archive: bogus Xref: header");
+                warn("bogus Xref header");
 		SMfreearticle(art);
 		continue;
 	    }
 
 	    /* Process each newsgroup... */
 	    if (base) {
-		DISPOSE(base);
+		free(base);
 		base = NULL;
 	    }
-	    for (i=0; i<numxrefs; i++) {
+	    for (i=0; (unsigned)i<numxrefs; i++) {
 		/* Check for group limits... -p flag */
 		if ((p=strchr(xrefs[i], ':')) == NULL) {
-		    fprintf(stderr, "archive: bogus xref '%s'\n", xrefs[i]);
+                    warn("bogus Xref entry %s", xrefs[i]);
 		    continue;	/* Skip to next xref */
 		}
 		if (numgroups > 0) {
 		    *p = '\0';
 		    ng = xrefs[i];
-		    doit = FALSE;
-		    for (j=0; j<numgroups && !doit; j++) {
-			if (wildmat(ng, groups[j]) != 0) doit=TRUE;
+		    doit = false;
+		    for (j=0; (unsigned)j<numgroups && !doit; j++) {
+			if (uwildmat(ng, groups[j]) != 0) doit=true;
 		    }
 		}
 		else {
-		    doit = TRUE;
+		    doit = true;
 		}
 		*p = '/';
 		if (doit) {
@@ -601,14 +551,13 @@ main(ac, av)
 			*p = '\0';
 		    }
 			
-		    if (base) {
+		    if (base && !Concat) {
 			/* Try to link the file into the archive. */
 			if (link(base, dest) < 0) {
 
 			    /* Make the archive directory. */
 			    if (!MakeArchiveDirectory(dest)) {
-				(void)fprintf(stderr, "Can't mkdir for \"%s\", %s\n",
-					dest, strerror(errno));
+                                syswarn("cannot mkdir for %s", dest);
 				continue;
 			    }
 
@@ -616,8 +565,8 @@ main(ac, av)
 			    if (link(base, dest) < 0) {
 #if	defined(HAVE_SYMLINK)
 				if (symlink(base, dest) < 0)
-				    (void)fprintf(stderr, "Can't symlink \"%s\" to \"%s\", %s\n",
-					    dest, base, strerror(errno));
+                                    syswarn("cannot symlink %s to %s",
+                                            dest, base);
 				else
 #endif	/* defined(HAVE_SYMLINK) */
 				if (!Copy(base, dest))
@@ -626,19 +575,16 @@ main(ac, av)
 			    }
 			}
 		    } else {
-			if (!CopyArt(art, dest, Concat)) {
-			    fprintf(stderr, 
-				"archive: %s->%s failed\n", buff, dest);
-			}
-			base = COPY(dest);
+			if (!CopyArt(art, dest, Concat))
+                            syswarn("copying %s to %s failed", buff, dest);
+			base = xstrdup(dest);
 		    }
 
 	            /* Write index. */
 	            if (Index) {
-	                WriteArtIndex(art, dest, Name);
+	                WriteArtIndex(art, Name);
 	                if (ferror(stdout) || fflush(stdout) == EOF)
-		            (void)fprintf(stderr, "Can't write index for \"%s\", %s\n",
-			            Name, strerror(errno));
+                            syswarn("cannot write index for %s", Name);
 	            }
 		}
 	    }
@@ -647,12 +593,12 @@ main(ac, av)
 	    SMfreearticle(art);
 	    art = NULL;
 	    /* Free up the xrefs storage space */
-	    for ( i=0; i<numxrefs; i++) DISPOSE(xrefs[i]);
-	    DISPOSE(xrefs);
+	    for ( i=0; (unsigned)i<numxrefs; i++) free(xrefs[i]);
+	    free(xrefs);
 	    numxrefs = 0;
 	    xrefs = NULL;
 	} else {
-	    (void)fprintf(stderr, "not token %s\n", buff);
+            warn("%s is not a token", buff);
 	    continue;
 	}
     }
@@ -662,50 +608,43 @@ main(ac, av)
 
     /* If we read all our input, try to remove the file, and we're done. */
     if (feof(stdin)) {
-	(void)fclose(stdin);
+	fclose(stdin);
 	if (av[0])
-	    (void)unlink(av[0]);
+	    unlink(av[0]);
 	exit(0);
     }
 
     /* Make an appropriate spool file. */
     p = av[0];
     if (p == NULL)
-	(void)sprintf(temp, "%s/%s", innconf->pathoutgoing, "archive");
+        spool = concatpath(innconf->pathoutgoing, "archive");
     else if (*p == '/')
-	(void)sprintf(temp, "%s.bch", p);
+        spool = concat(p, ".bch", (char *) 0);
     else
-	(void)sprintf(temp, "%s/%s.bch", innconf->pathoutgoing, p);
-    if ((F = xfopena(temp)) == NULL) {
-	(void)fprintf(stderr, "archive: Can't spool to \"%s\", %s\n",
-	    temp, strerror(errno));
-	exit(1);
-    }
+        spool = concat(innconf->pathoutgoing, "/", p, ".bch", (char *) 0);
+    if ((F = xfopena(spool)) == NULL)
+        sysdie("cannot spool to %s", spool);
 
     /* Write the rest of stdin to the spool file. */
     i = 0;
     if (fprintf(F, "%s\n", buff) == EOF) {
-	(void)fprintf(stderr, "archive:  Can't start spool, %s\n",
-		strerror(errno));
+        syswarn("cannot start spool");
 	i = 1;
     }
     while (fgets(buff, sizeof buff, stdin) != NULL) 
 	if (fputs(buff, F) == EOF) {
-	    (void)fprintf(stderr, "archive: Can't write spool, %s\n",
-		    strerror(errno));
+            syswarn("cannot write to spool");
 	    i = 1;
 	    break;
 	}
     if (fclose(F) == EOF) {
-	(void)fprintf(stderr, "archive: Can't close spool, %s\n",
-	    strerror(errno));
+        syswarn("cannot close spool");
 	i = 1;
     }
 
     /* If we had a named input file, try to rename the spool. */
-    if (p != NULL && rename(temp, av[0]) < 0) {
-	(void)fprintf(stderr, "archive: Can't rename spool, %s\n",
-	    strerror(errno));
+    if (p != NULL && rename(spool, av[0]) < 0) {
+        syswarn("cannot rename spool");
 	i = 1;
     }
 
