@@ -1,8 +1,8 @@
-/*  $Revision: 1.2 $
+/*  $Id: encode.c,v 1.7 2003/01/13 07:59:30 rra Exp $
+**
 **  Produce a seven-bit printable encoding of stdin on stdout.
 **  From @(#)encode.c 1.3 5/15/85, distributed with B2.11 News.
 **
-**
 **  The encoding uses characters from 0x20 (' ') through 0x7A ('z').
 **  (That fits nicely into the UUCP 'f' protocol by Piet Beertema.) First,
 **  expand three eight-bit charcters into four six-bit ones.  Collect
@@ -15,9 +15,8 @@
 **  And if you thought that was unclear, then we won't even get into the
 **  terminating conditions!
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 
 
@@ -35,12 +34,10 @@ static int	Count;
 
 
 static void
-dumpcode(p, n)
-    register char	*p;
-    register int	n;
+dumpcode(char *p, int n)
 {
-    register int	last;
-    register int	c;
+    int	last;
+    int	c;
 
     if (n == 13) {
 	n--;
@@ -58,29 +55,27 @@ dumpcode(p, n)
 	    c |= (1 << 12);
 	last <<= 1;
 
-	(void)putchar((c / 91) + ' ');
-	(void)putchar((c % 91) + ' ');
+	putchar((c / 91) + ' ');
+	putchar((c % 91) + ' ');
     }
 }
 
 static void
-flushout()
+flushout(void)
 {
-    (void)putchar(ENDMARK1);
-    (void)putchar(ENDMARK2);
-    (void)putchar(Count + ' ');
+    putchar(ENDMARK1);
+    putchar(ENDMARK2);
+    putchar(Count + ' ');
     dumpcode(Buffer, Count);
 }
 
 
 static void
-encode(dest, n)
-    register char	*dest;
-    int			n;
+encode(char *dest, int n)
 {
-    register char	*p;
-    register int	i;
-    register int	j;
+    char	*p;
+    int	i;
+    int	j;
     char		b4[4];
 
     b4[0] = (dest[0] >> 2) & 0x3F;
@@ -100,14 +95,11 @@ encode(dest, n)
 }
 
 
-/* ARGSUSED0 */
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(void)
 {
-    register char	*p;
-    register int	c;
+    char	*p;
+    int	c;
     char		b3[3];
 
     for (p = b3; (c = getchar()) != EOF; ) {
