@@ -1,37 +1,37 @@
-/*  $Id: site.c,v 1.22.2.1 1999/04/19 06:54:46 kondou Exp $
+/*  $Id: site.c,v 1.49.2.2 2004/05/16 20:26:54 rra Exp $
 **
 **  Routines to implement site-feeding.  Mainly working with channels to
 **  do buffering and determine what to send.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
+
+#include "inn/innconf.h"
 #include "innd.h"
 
 
-STATIC int	SITEcount;
-STATIC int	SITEhead = NOSITE;
-STATIC int	SITEtail = NOSITE;
-STATIC char	SITEshell[] = _PATH_SH;
+static int	SITEcount;
+static int	SITEhead = NOSITE;
+static int	SITEtail = NOSITE;
+static char	SITEshell[] = _PATH_SH;
 
 
 /*
 **  Called when input is ready to read.  Shouldn't happen.
 */
-/* ARGSUSED0 */
-STATIC FUNCTYPE SITEreader(CHANNEL *cp)
+static void
+SITEreader(CHANNEL *cp)
 {
-    syslog(L_ERROR, "%s internal SITEreader", LogName);
+    syslog(L_ERROR, "%s internal SITEreader: %s", LogName, CHANname(cp));
 }
 
 
 /*
 **  Called when write is done.  No-op.
 */
-/* ARGSUSED0 */
-STATIC FUNCTYPE SITEwritedone(CHANNEL *cp)
+static void
+SITEwritedone(CHANNEL *cp UNUSED)
 {
 }
 
@@ -39,7 +39,8 @@ STATIC FUNCTYPE SITEwritedone(CHANNEL *cp)
 /*
 **  Make a site start spooling.
 */
-BOOL SITEspool(SITE *sp, CHANNEL *cp)
+static bool
+SITEspool(SITE *sp, CHANNEL *cp)
 {
     int			i;
     char		buff[SPOOLNAMEBUFF];
@@ -57,10 +58,8 @@ BOOL SITEspool(SITE *sp, CHANNEL *cp)
 	syslog(L_ERROR, "%s cant open %s %m", sp->Name, name);
 	IOError("site batch file", i);
 	sp->Channel = NULL;
-	return FALSE;
+	return false;
     }
-    if (AmRoot)
-	xchown(name);
     if (cp) {
       if (cp->fd >= 0)
         /* syslog(L_ERROR, "DEBUG ERROR SITEspool trashed:%d %s:%d", cp->fd, sp->Name, i);
@@ -68,19 +67,19 @@ BOOL SITEspool(SITE *sp, CHANNEL *cp)
 	WCHANremove(cp);
 	RCHANremove(cp);
 	SCHANremove(cp);
-	(void)close(cp->fd);
+	close(cp->fd);
 	cp->fd = i;
-	return TRUE;
+	return true;
     }
     sp->Channel = CHANcreate(i, CTfile, CSwriting, SITEreader, SITEwritedone);
     if (sp->Channel == NULL) {
 	syslog(L_ERROR, "%s cant channel %m", sp->Name);
-	(void)close(i);
-	return FALSE;
+	close(i);
+	return false;
     }
     WCHANset(sp->Channel, "", 0);
-    sp->Spooling = TRUE;
-    return TRUE;
+    sp->Spooling = true;
+    return true;
 }
 
 
@@ -88,7 +87,8 @@ BOOL SITEspool(SITE *sp, CHANNEL *cp)
 **  Delete a site from the file writing list.  Can be called even if
 **  site is not on the list.
 */
-static void SITEunlink(SITE *sp)
+static void
+SITEunlink(SITE *sp)
 {
     if (sp->Next != NOSITE || sp->Prev != NOSITE
      || (SITEhead != NOSITE && sp == &Sites[SITEhead]))
@@ -111,11 +111,12 @@ static void SITEunlink(SITE *sp)
 /*
 **  Find the oldest "file feed" site and buffer it.
 */
-STATIC void SITEbufferoldest(void)
+static void
+SITEbufferoldest(void)
 {
     SITE	        *sp;
-    BUFFER	        *bp;
-    BUFFER	        *out;
+    struct buffer       *bp;
+    struct buffer       *out;
 
     /* Get the oldest user of a file. */
     if (SITEtail == NOSITE) {
@@ -140,27 +141,27 @@ STATIC void SITEbufferoldest(void)
     }
 
     /* Write out what we can. */
-    (void)WCHANflush(sp->Channel);
+    WCHANflush(sp->Channel);
 
     /* Get a buffer for the site. */
-    sp->Buffered = TRUE;
+    sp->Buffered = true;
     bp = &sp->Buffer;
-    bp->Used = 0;
-    bp->Left = 0;
-    if (bp->Size == 0) {
-	bp->Size = sp->Flushpoint;
-	bp->Data = NEW(char, bp->Size);
+    bp->used = 0;
+    bp->left = 0;
+    if (bp->size == 0) {
+	bp->size = sp->Flushpoint;
+	bp->data = xmalloc(bp->size);
     }
     else {
-	bp->Size = sp->Flushpoint;
-	RENEW(bp->Data, char, bp->Size);
+	bp->size = sp->Flushpoint;
+        bp->data = xrealloc(bp->data, bp->size);
     }
 
     /* If there's any unwritten data, copy it. */
     out = &sp->Channel->Out;
-    if (out->Left) {
-	BUFFset(bp, &out->Data[out->Used], out->Left);
-	out->Left = 0;
+    if (out->left) {
+        buffer_set(bp, &out->data[out->used], out->left);
+	out->left = 0;
     }
 
     /* Now close the original channel. */
@@ -171,7 +172,8 @@ STATIC void SITEbufferoldest(void)
 /*
  * *  Bilge Site's Channel out buffer.
  */
-STATIC BOOL SITECHANbilge(SITE *sp)
+static bool
+SITECHANbilge(SITE *sp)
 {
     int             fd;
     int             i;
@@ -190,41 +192,42 @@ STATIC BOOL SITECHANbilge(SITE *sp)
         syslog(L_ERROR, "%s cant open %s %m", sp->Name, name);
         IOError("site batch file",oerrno);
         sp->Channel = NULL;
-        return FALSE;
+        return false;
     }
-    while(sp->Channel->Out.Left > 0) {
-        i = write(fd, (POINTER) &sp->Channel->Out.Data[sp->Channel->Out.Used],
-            (SIZE_T) sp->Channel->Out.Left);
+    while (sp->Channel->Out.left > 0) {
+        i = write(fd, &sp->Channel->Out.data[sp->Channel->Out.used],
+                  sp->Channel->Out.left);
         if(i <= 0) {
-            syslog(L_ERROR,"%s cant spool count %d",CHANname(sp->Channel),
-                sp->Channel->Out.Left);
+            syslog(L_ERROR,"%s cant spool count %lu", CHANname(sp->Channel),
+                (unsigned long) sp->Channel->Out.left);
             close(fd);
-            return FALSE;
+            return false;
         }
-        sp->Channel->Out.Left -= i;
-        sp->Channel->Out.Used += i;
+        sp->Channel->Out.left -= i;
+        sp->Channel->Out.used += i;
     }
     close(fd);
-    DISPOSE(sp->Channel->Out.Data);
-    sp->Channel->Out.Data = NEW(char, SMBUF);
-    sp->Channel->Out.Size = SMBUF;
-    sp->Channel->Out.Left = 0;
-    sp->Channel->Out.Used = 0;
-    return TRUE;
+    free(sp->Channel->Out.data);
+    sp->Channel->Out.data = xmalloc(SMBUF);
+    sp->Channel->Out.size = SMBUF;
+    sp->Channel->Out.left = 0;
+    sp->Channel->Out.used = 0;
+    return true;
 }
 
 /*
 **  Check if we need to write out the site's buffer.  If we're buffered
 **  or the feed is backed up, this gets a bit complicated.
 */
-STATIC void SITEflushcheck(SITE *sp, BUFFER *bp)
+static void
+SITEflushcheck(SITE *sp, struct buffer *bp)
 {
     int	                i;
     CHANNEL	        *cp;
 
     /* If we're buffered, and we hit the flushpoint, do an LRU. */
     if (sp->Buffered) {
-	if (bp->Left < sp->Flushpoint)
+	if (bp->left < sp->Flushpoint)
 	    return;
 	while (SITEcount >= MaxOutgoing)
 	    SITEbufferoldest();
@@ -235,7 +238,7 @@ STATIC void SITEflushcheck(SITE *sp, BUFFER *bp)
 	WCHANsetfrombuffer(sp->Channel, bp);
 	WCHANadd(sp->Channel);
 	/* Reset buffer; data has been moved. */
-	BUFFset(bp, "", 0);
+	buffer_set(bp, "", 0);
     }
 
     if (PROCneedscan)
@@ -243,29 +246,29 @@ STATIC void SITEflushcheck(SITE *sp, BUFFER *bp)
 
     /* Handle buffering. */
     cp = sp->Channel;
-    i = cp->Out.Left;
+    i = cp->Out.left;
     if (i < sp->StopWriting)
 	WCHANremove(cp);
     if ((sp->StartWriting == 0 || i > sp->StartWriting)
      && !CHANsleeping(cp)) {
 	if (sp->Type == FTchannel) {	/* channel feed, try the write */
 	    int j;
-	    if (bp->Left == 0)
+	    if (bp->left == 0)
 		return;
-	    j = write(cp->fd, (POINTER)&bp->Data[bp->Used], (SIZE_T)bp->Left);
+	    j = write(cp->fd, &bp->data[bp->used], bp->left);
 	    if (j > 0) {
-		bp->Left -= j;
-		bp->Used += j;
-		i = cp->Out.Left;
+		bp->left -= j;
+		bp->used += j;
+		i = cp->Out.left;
 		/* Since we just had a successful write, we need to clear the 
 		 * channel's error counts. - dave@jetcafe.org */
 		cp->BadWrites = 0;
 		cp->BlockedWrites = 0;
 	    }
-	    if (bp->Left <= 0) {
+	    if (bp->left <= 0) {
                 /* reset Used, Left on bp, keep channel buffer size from
                    exploding. */
-                bp->Used = bp->Left = 0;
+                bp->used = bp->left = 0;
 		WCHANremove(cp);
             } else
 		WCHANadd(cp);
@@ -291,10 +294,11 @@ STATIC void SITEflushcheck(SITE *sp, BUFFER *bp)
 /*
 **  Send a control line to an exploder.
 */
-void SITEwrite(SITE *sp, STRING text)
+void
+SITEwrite(SITE *sp, const char *text)
 {
     static char		PREFIX[] = { EXP_CONTROL, '\0' };
-    BUFFER	        *bp;
+    struct buffer       *bp;
 
     if (sp->Buffered)
 	bp = &sp->Buffer;
@@ -304,9 +308,9 @@ void SITEwrite(SITE *sp, STRING text)
 	sp->Channel->LastActive = Now.time;
 	bp = &sp->Channel->Out;
     }
-    BUFFappend(bp, PREFIX, STRLEN(PREFIX));
-    BUFFappend(bp, text, (int)strlen(text));
-    BUFFappend(bp, "\n", 1);
+    buffer_append(bp, PREFIX, strlen(PREFIX));
+    buffer_append(bp, text, strlen(text));
+    buffer_append(bp, "\n", 1);
     if (sp->Channel != NULL)
 	WCHANadd(sp->Channel);
 }
@@ -315,14 +319,16 @@ void SITEwrite(SITE *sp, STRING text)
 /*
 **  Send the desired data about an article down a channel.
 */
-STATIC void SITEwritefromflags(SITE *sp, ARTDATA *Data)
+static void
+SITEwritefromflags(SITE *sp, ARTDATA *Data)
 {
+    HDRCONTENT		*hc = Data->HdrContent;
     static char		ITEMSEP[] = " ";
     static char		NL[] = "\n";
     char		pbuff[12];
     char	        *p;
-    BOOL	        Dirty;
-    BUFFER	        *bp;
+    bool	        Dirty;
+    struct buffer       *bp;
     SITE	        *spx;
     int	                i;
 
@@ -336,123 +342,130 @@ STATIC void SITEwritefromflags(SITE *sp, ARTDATA *Data)
 	    return;
 	bp = &sp->Channel->Out;
     }
-    for (Dirty = FALSE, p = sp->FileFlags; *p; p++) {
+    for (Dirty = false, p = sp->FileFlags; *p; p++) {
 	switch (*p) {
 	default:
 	    syslog(L_ERROR, "%s internal SITEwritefromflags %c", sp->Name, *p);
 	    continue;
 	case FEED_BYTESIZE:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Size, Data->SizeLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, Data->Bytes + sizeof("Bytes: ") - 1,
+                          Data->BytesLength);
 	    break;
 	case FEED_FULLNAME:
+	case FEED_NAME:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    if (!innconf->storageapi) {
-		BUFFappend(bp, innconf->patharticles, SPOOLlen);
-		BUFFappend(bp, "/", 1);
-	    }
-	    BUFFappend(bp, Data->Name, Data->NameLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, Data->TokenText, sizeof(TOKEN) * 2 + 2);
 	    break;
 	case FEED_HASH:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, "[", 1);
-	    BUFFappend(bp, HashToText(*(Data->Hash)), sizeof(HASH)*2);
-	    BUFFappend(bp, "]", 1);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, "[", 1);
+	    buffer_append(bp, HashToText(*(Data->Hash)), sizeof(HASH)*2);
+	    buffer_append(bp, "]", 1);
 	    break;
 	case FEED_HDR_DISTRIB:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Distribution, Data->DistributionLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, HDR(HDR__DISTRIBUTION),
+                          HDR_LEN(HDR__DISTRIBUTION));
 	    break;
 	case FEED_HDR_NEWSGROUP:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Newsgroups, Data->NewsgroupsLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, HDR(HDR__NEWSGROUPS), HDR_LEN(HDR__NEWSGROUPS));
 	    break;
 	case FEED_HEADERS:
 	    if (Dirty)
-		BUFFappend(bp, NL, STRLEN(NL));
-	    BUFFappend(bp, Data->Headers->Data, Data->Headers->Left);
+		buffer_append(bp, NL, strlen(NL));
+	    buffer_append(bp, Data->Headers.data, Data->Headers.left);
 	    break;
 	case FEED_OVERVIEW:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Overview->Data, Data->Overview->Left);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, Data->Overview.data, Data->Overview.left);
 	    break;
 	case FEED_PATH:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    if (!Hassamepath)
-		BUFFappend(bp, Path.Data, Path.Used);
-	    if (AddAlias)
-		BUFFappend(bp, Pathalias.Data, Pathalias.Used);
-	    BUFFappend(bp, Data->Path, Data->PathLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    if (!Data->Hassamepath)
+		buffer_append(bp, Path.data, Path.used);
+	    if (Data->AddAlias)
+		buffer_append(bp, Pathalias.data, Pathalias.used);
+	    buffer_append(bp, HDR(HDR__PATH), HDR_LEN(HDR__PATH));
 	    break;
 	case FEED_REPLIC:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Replic, Data->ReplicLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, Data->Replic, Data->ReplicLength);
+	    break;
+	case FEED_STOREDGROUP:
+	    if (Dirty)
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, Data->Newsgroups.List[0],
+                          Data->StoredGroupLength);
 	    break;
 	case FEED_TIMERECEIVED:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->TimeReceived, Data->TimeReceivedLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    snprintf(pbuff, sizeof(pbuff), "%ld", (long) Data->Arrived);
+	    buffer_append(bp, pbuff, strlen(pbuff));
 	    break;
 	case FEED_TIMEPOSTED:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    sprintf(pbuff, "%ld", Data->Posted);
-	    BUFFappend(bp, pbuff, strlen(pbuff));
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    snprintf(pbuff, sizeof(pbuff), "%ld", (long) Data->Posted);
+	    buffer_append(bp, pbuff, strlen(pbuff));
+	    break;
+	case FEED_TIMEEXPIRED:
+	    if (Dirty)
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    snprintf(pbuff, sizeof(pbuff), "%ld", (long) Data->Expires);
+	    buffer_append(bp, pbuff, strlen(pbuff));
 	    break;
 	case FEED_MESSAGEID:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->MessageID, Data->MessageIDLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, HDR(HDR__MESSAGE_ID), HDR_LEN(HDR__MESSAGE_ID));
 	    break;
 	case FEED_FNLNAMES:
-	    if (sp->FNLnames.Data) {
+	    if (sp->FNLnames.data) {
 		/* Funnel; write names of our sites that got it. */
 		if (Dirty)
-		    BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-		BUFFappend(bp, sp->FNLnames.Data, sp->FNLnames.Used);
+		    buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+		buffer_append(bp, sp->FNLnames.data, sp->FNLnames.used);
 	    }
 	    else {
 		/* Not funnel; write names of all sites that got it. */
 		for (spx = Sites, i = nSites; --i >= 0; spx++)
 		    if (spx->Sendit) {
 			if (Dirty)
-			    BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-			BUFFappend(bp, spx->Name, spx->NameLength);
-			Dirty = TRUE;
+			    buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+			buffer_append(bp, spx->Name, spx->NameLength);
+			Dirty = true;
 		    }
 	    }
 	    break;
-	case FEED_NAME:
-	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Name, Data->NameLength);
-	    break;
 	case FEED_NEWSGROUP:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
 	    if (sp->ng)
-		BUFFappend(bp, sp->ng->Name, sp->ng->NameLength);
+		buffer_append(bp, sp->ng->Name, sp->ng->NameLength);
 	    else
-		BUFFappend(bp, "?", 1);
+		buffer_append(bp, "?", 1);
 	    break;
 	case FEED_SITE:
 	    if (Dirty)
-		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, Data->Feedsite, Data->FeedsiteLength);
+		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
+	    buffer_append(bp, Data->Feedsite, Data->FeedsiteLength);
 	    break;
 	}
-	Dirty = TRUE;
+	Dirty = true;
     }
     if (Dirty) {
-	BUFFappend(bp, "\n", 1);
+	buffer_append(bp, "\n", 1);
 	SITEflushcheck(sp, bp);
     }
 }
@@ -461,14 +474,14 @@ STATIC void SITEwritefromflags(SITE *sp, ARTDATA *Data)
 /*
 **  Send one article to a site.
 */
-void SITEsend(SITE *sp, ARTDATA *Data)
+void
+SITEsend(SITE *sp, ARTDATA *Data)
 {
     int	                i;
     char	        *p;
     char		*temp;
     char		buff[BUFSIZ];
-    STRING		argv[MAX_BUILTIN_ARGV];
-    int			fd;
+    char *		argv[MAX_BUILTIN_ARGV];
 
     switch (sp->Type) {
     default:
@@ -487,50 +500,37 @@ void SITEsend(SITE *sp, ARTDATA *Data)
     case FTprogram:
 	/* Set up the argument vector. */
 	if (sp->FNLwantsnames) {
-	    i = strlen(sp->Param) + sp->FNLnames.Used;
-	    if (i + strlen(Data->Name) >= sizeof buff) {
-		syslog(L_ERROR, "%s toolong need %d for %s",
-		    sp->Name, i + strlen(Data->Name), Data->Name);
+	    i = strlen(sp->Param) + sp->FNLnames.used;
+	    if (i + (sizeof(TOKEN) * 2) + 3 >= sizeof buff) {
+		syslog(L_ERROR, "%s toolong need %lu for %s",
+		    sp->Name, (unsigned long) (i + (sizeof(TOKEN) * 2) + 3),
+		    sp->Name);
 		break;
 	    }
-	    temp = NEW(char, i + 1);
+	    temp = xmalloc(i + 1);
 	    p = strchr(sp->Param, '*');
 	    *p = '\0';
-	    (void)strcpy(temp, sp->Param);
-	    (void)strcat(temp, sp->FNLnames.Data);
-	    (void)strcat(temp, &p[1]);
+	    strlcpy(temp, sp->Param, i + 1);
+	    strlcat(temp, sp->FNLnames.data, i + 1);
+	    strlcat(temp, &p[1], i + 1);
 	    *p = '*';
-	    (void)sprintf(buff, temp, Data->Name);
-	    DISPOSE(temp);
+	    snprintf(buff, sizeof(buff), temp, Data->TokenText);
+	    free(temp);
 	}
 	else
-	    (void)sprintf(buff, sp->Param, Data->Name);
+	    snprintf(buff, sizeof(buff), sp->Param, Data->TokenText);
 
-	if (NeedShell(buff, argv, ENDOF(argv))) {
+	if (NeedShell(buff, (const char **)argv, (const char **)ARRAY_END(argv))) {
 	    argv[0] = SITEshell;
-	    argv[1] = "-c";
+	    argv[1] = (char *) "-c";
 	    argv[2] = buff;
 	    argv[3] = NULL;
 	}
 
-	/* Feed the article on standard input. */
-	if (IsToken(Data->Name))
-	    fd = 0;
-	else
-	    fd = open(Data->Name, O_RDONLY);
-	if (fd < 0) {
-	    /* Unlikely, but we could check if the article is cross-posted
-	     * and try it under other names... */
-	    syslog(L_ERROR, "%s cant open %s %m", sp->Name, Data->Name);
-	    fd = 0;
-	}
-
 	/* Start the process. */
-	i = Spawn(sp->Nice, fd, (int)fileno(Errlog), (int)fileno(Errlog), argv);
+	i = Spawn(sp->Nice, 0, (int)fileno(Errlog), (int)fileno(Errlog), argv);
 	if (i >= 0)
-	    (void)PROCwatch(i, -1);
-	if (fd != 0)
-	    (void)close(fd);
+	    PROCwatch(i, -1);
 	break;
     }
 }
@@ -540,14 +540,15 @@ void SITEsend(SITE *sp, ARTDATA *Data)
 **  The channel was sleeping because we had to spool our output to
 **  a file.  Flush and restart.
 */
-STATIC FUNCTYPE SITEspoolwake(CHANNEL *cp)
+static void
+SITEspoolwake(CHANNEL *cp)
 {
     SITE	*sp;
     int		*ip;
 
-    ip = CAST(int*, cp->Argument);
+    ip = (int *) cp->Argument;
     sp = &Sites[*ip];
-    DISPOSE(cp->Argument);
+    free(cp->Argument);
     cp->Argument = NULL;
     if (sp->Channel != cp) {
 	syslog(L_ERROR, "%s internal SITEspoolwake %s got %d, not %d",
@@ -555,7 +556,7 @@ STATIC FUNCTYPE SITEspoolwake(CHANNEL *cp)
         return;
     }
     syslog(L_NOTICE, "%s spoolwake", sp->Name);
-    SITEflush(sp, TRUE);
+    SITEflush(sp, true);
 }
 
 
@@ -563,10 +564,11 @@ STATIC FUNCTYPE SITEspoolwake(CHANNEL *cp)
 **  Start up a process for a channel, or a spool to a file if we can't.
 **  Create a channel for the site to talk to.
 */
-STATIC BOOL SITEstartprocess(SITE *sp)
+static bool
+SITEstartprocess(SITE *sp)
 {
-    PID_T	        i;
-    STRING		argv[MAX_BUILTIN_ARGV];
+    pid_t	        i;
+    char		*argv[MAX_BUILTIN_ARGV];
     char		*process;
     int			*ip;
     int			pan[2];
@@ -575,79 +577,73 @@ STATIC BOOL SITEstartprocess(SITE *sp)
     /* Create a socketpair. */
     if (socketpair(PF_UNIX, SOCK_STREAM, 0, pan) < 0) {
 	syslog(L_ERROR, "%s cant socketpair %m", sp->Name);
-	return FALSE;
+	return false;
     }
 #else
     /* Create a pipe. */
     if (pipe(pan) < 0) {
 	syslog(L_ERROR, "%s cant pipe %m", sp->Name);
-	return FALSE;
+	return false;
     }
 #endif
-    CloseOnExec(pan[PIPE_WRITE], TRUE);
+    close_on_exec(pan[PIPE_WRITE], true);
 
     /* Set up the argument vector. */
-    process = COPY(sp->Param);
-    if (NeedShell(process, argv, ENDOF(argv))) {
+    process = xstrdup(sp->Param);
+    if (NeedShell(process, (const char **)argv, (const char **)ARRAY_END(argv))) {
 	argv[0] = SITEshell;
-	argv[1] = "-c";
+	argv[1] = (char *) "-c";
 	argv[2] = process;
 	argv[3] = NULL;
     }
 
     /* Fork a child. */
-    i = Spawn(sp->Nice, pan[PIPE_READ], (int)fileno(Errlog), (int)fileno(Errlog), argv);
+    i = Spawn(sp->Nice, pan[PIPE_READ], (int)fileno(Errlog),
+	      (int)fileno(Errlog), argv);
     if (i > 0) {
 	sp->pid = i;
-	sp->Spooling = FALSE;
+	sp->Spooling = false;
 	sp->Process = PROCwatch(i, sp - Sites);
-	(void)close(pan[PIPE_READ]);
+	close(pan[PIPE_READ]);
 	sp->Channel = CHANcreate(pan[PIPE_WRITE],
 			sp->Type == FTchannel ? CTprocess : CTexploder,
 			CSwriting, SITEreader, SITEwritedone);
-	DISPOSE(process);
-	return TRUE;
+	free(process);
+	return true;
     }
 
     /* Error.  Switch to spooling. */
     syslog(L_ERROR, "%s cant spawn spooling %m", sp->Name);
-    (void)close(pan[PIPE_WRITE]);
-    (void)close(pan[PIPE_READ]);
-    DISPOSE(process);
+    close(pan[PIPE_WRITE]);
+    close(pan[PIPE_READ]);
+    free(process);
     if (!SITEspool(sp, (CHANNEL *)NULL))
-	return FALSE;
+	return false;
 
     /* We'll try to restart the channel later. */
     syslog(L_ERROR, "%s cant spawn spooling %m", sp->Name);
-    ip = NEW(int, 1);
+    ip = xmalloc(sizeof(int));
     *ip = sp - Sites;
-    SCHANadd(sp->Channel, (time_t)(Now.time + innconf->chanretrytime),
-	(POINTER)NULL, SITEspoolwake, (POINTER)ip);
-    return TRUE;
+    SCHANadd(sp->Channel, Now.time + innconf->chanretrytime, NULL,
+             SITEspoolwake, ip);
+    return true;
 }
 
 
 /*
 **  Set up a site for internal buffering.
 */
-STATIC void SITEbuffer(SITE *sp)
+static void
+SITEbuffer(SITE *sp)
 {
-    BUFFER	        *bp;
+    struct buffer *bp;
 
     SITEunlink(sp);
-    sp->Buffered = TRUE;
+    sp->Buffered = true;
     sp->Channel = NULL;
     bp = &sp->Buffer;
-    if (bp->Size == 0) {
-	bp->Size = sp->Flushpoint;
-	bp->Data = NEW(char, bp->Size);
-	BUFFset(bp, "", 0);
-    }
-    else if (bp->Size < sp->Flushpoint) {
-	bp->Size = sp->Flushpoint;
-	RENEW(bp->Data, char, bp->Size);
-    }
-    BUFFset(bp, "", 0);
+    buffer_resize(bp, sp->Flushpoint);
+    buffer_set(bp, "", 0);
     syslog(L_NOTICE, "%s buffered", sp->Name);
 }
 
@@ -655,7 +651,8 @@ STATIC void SITEbuffer(SITE *sp)
 /*
 **  Link a site at the head of the "currently writing to a file" list
 */
-static void SITEmovetohead(SITE *sp)
+static void
+SITEmovetohead(SITE *sp)
 {
     if ((SITEhead == NOSITE) && ((sp->Next != NOSITE) || (sp->Prev != NOSITE)))
 	SITEunlink(sp);
@@ -675,7 +672,8 @@ static void SITEmovetohead(SITE *sp)
 /*
 **  Set up a site's feed.  This means opening a file or channel if needed.
 */
-BOOL SITEsetup(SITE *sp)
+bool
+SITEsetup(SITE *sp)
 {
     int			fd;
     int			oerrno;
@@ -684,7 +682,7 @@ BOOL SITEsetup(SITE *sp)
     default:
 	syslog(L_ERROR, "%s internal SITEsetup %d",
 	    sp->Name, sp->Type);
-	return FALSE;
+	return false;
     case FTfunnel:
     case FTlogonly:
     case FTprogram:
@@ -694,7 +692,7 @@ BOOL SITEsetup(SITE *sp)
 	if (SITEcount >= MaxOutgoing)
 	    SITEbuffer(sp);
 	else {
-	    sp->Buffered = FALSE;
+	    sp->Buffered = false;
 	    fd = open(sp->Param, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
 	    if (fd < 0) {
 		if (errno == EMFILE) {
@@ -705,11 +703,9 @@ BOOL SITEsetup(SITE *sp)
 		oerrno = errno;
 		syslog(L_NOTICE, "%s cant open %s %m", sp->Name, sp->Param);
 		IOError("site file", oerrno);
-		return FALSE;
+		return false;
 	    }
 	    SITEmovetohead(sp);
-	    if (AmRoot)
-		xchown(sp->Param);
 	    sp->Channel = CHANcreate(fd, CTfile, CSwriting,
 			    SITEreader, SITEwritedone);
 	    syslog(L_NOTICE, "%s opened %s", sp->Name, CHANname(sp->Channel));
@@ -719,24 +715,25 @@ BOOL SITEsetup(SITE *sp)
     case FTchannel:
     case FTexploder:
 	if (!SITEstartprocess(sp))
-	    return FALSE;
+	    return false;
 	syslog(L_NOTICE, "%s spawned %s", sp->Name, CHANname(sp->Channel));
 	WCHANset(sp->Channel, "", 0);
 	WCHANadd(sp->Channel);
 	break;
     }
-    return TRUE;
+    return true;
 }
 
 
 /*
 **  A site's channel process died; restart it.
 */
-void SITEprocdied(SITE *sp, int process, PROCESS *pp)
+void
+SITEprocdied(SITE *sp, int process, PROCESS *pp)
 {
     syslog(pp->Status ? L_ERROR : L_NOTICE, "%s exit %d elapsed %ld pid %ld",
 	sp->Name ? sp->Name : "?", pp->Status,
-	pp->Collected - pp->Started, (long)pp->Pid);
+	(long) (pp->Collected - pp->Started), (long) pp->Pid);
     if (sp->Process != process || sp->Name == NULL)
 	/* We already started a new process for this channel
 	 * or this site has been dropped. */
@@ -754,7 +751,8 @@ void SITEprocdied(SITE *sp, int process, PROCESS *pp)
 /*
 **  A channel is about to be closed; see if any site cares.
 */
-void SITEchanclose(CHANNEL *cp)
+void
+SITEchanclose(CHANNEL *cp)
 {
     int	                i;
     SITE	        *sp;
@@ -766,15 +764,16 @@ void SITEchanclose(CHANNEL *cp)
 	     * site spooling, copy any data that might be pending,
 	     * and arrange to retry later. */
 	    if (!SITEspool(sp, (CHANNEL *)NULL)) {
-		syslog(L_ERROR, "%s loss %d bytes", sp->Name, cp->Out.Left);
+		syslog(L_ERROR, "%s loss %lu bytes", sp->Name,
+                       (unsigned long) cp->Out.left);
 		return;
 	    }
 	    WCHANsetfrombuffer(sp->Channel, &cp->Out);
 	    WCHANadd(sp->Channel);
-	    ip = NEW(int, 1);
+	    ip = xmalloc(sizeof(int));
 	    *ip = sp - Sites;
-	    SCHANadd(sp->Channel, (time_t)(Now.time + innconf->chanretrytime),
-		(POINTER)NULL, SITEspoolwake, (POINTER)ip);
+	    SCHANadd(sp->Channel, Now.time + innconf->chanretrytime, NULL,
+                     SITEspoolwake, ip);
 	    break;
 	}
 }
@@ -783,10 +782,11 @@ void SITEchanclose(CHANNEL *cp)
 /*
 **  Flush any pending data waiting to be sent.
 */
-void SITEflush(SITE *sp, BOOL Restart)
+void
+SITEflush(SITE *sp, const bool Restart)
 {
     CHANNEL	        *cp;
-    BUFFER	        *out;
+    struct buffer       *out;
     int			count;
 
     if (sp->Name == NULL)
@@ -810,7 +810,7 @@ void SITEflush(SITE *sp, BOOL Restart)
     case FTexploder:
 	/* If spooling, close the file right now -- documented behavior. */
 	if (sp->Spooling && (cp = sp->Channel) != NULL) {
-	    (void)WCHANflush(cp);
+	    WCHANflush(cp);
 	    CHANclose(cp, CHANname(cp));
 	    sp->Channel = NULL;
 	}
@@ -826,7 +826,7 @@ void SITEflush(SITE *sp, BOOL Restart)
 	    if (!SITEsetup(sp) || sp->Buffered)
 		syslog(L_ERROR, "%s cant unbuffer to flush", sp->Name);
 	    else
-		BUFFswap(&sp->Buffer, &sp->Channel->Out);
+		buffer_swap(&sp->Buffer, &sp->Channel->Out);
 	    SITEcount += count;
 	}
 	break;
@@ -834,7 +834,7 @@ void SITEflush(SITE *sp, BOOL Restart)
 
     /* We're only dealing with files and channels now. */
     if ((cp = sp->Channel) != NULL)
-	(void)WCHANflush(cp);
+	WCHANflush(cp);
 
     /* Restart the site, copy any pending data. */
     if (Restart) {
@@ -844,23 +844,25 @@ void SITEflush(SITE *sp, BOOL Restart)
 	    if (sp->Buffered) {
 		/* SITEsetup had to buffer us; save any residue. */
 		out = &cp->Out;
-	        if (out->Left)
-		    BUFFset(&sp->Buffer, &out->Data[out->Used], out->Left);
+	        if (out->left)
+		    buffer_set(&sp->Buffer, &out->data[out->used], out->left);
 	    }
 	    else
 		WCHANsetfrombuffer(sp->Channel, &cp->Out);
 	}
     }
-    else if (cp != NULL && cp->Out.Left) {
+    else if (cp != NULL && cp->Out.left) {
 	if (sp->Type == FTfile || sp->Spooling) {
 	    /* Can't flush a file?  Hopeless. */
-	    syslog(L_ERROR, "%s dataloss %d", sp->Name, cp->Out.Left);
+	    syslog(L_ERROR, "%s dataloss %lu", sp->Name,
+                   (unsigned long) cp->Out.left);
 	    return;
 	}
 	/* Must be a working channel; spool and retry. */
-	syslog(L_ERROR, "%s spooling %d bytes", sp->Name, cp->Out.Left);
+	syslog(L_ERROR, "%s spooling %lu bytes", sp->Name,
+               (unsigned long) cp->Out.left);
 	if (SITEspool(sp, cp))
-	    SITEflush(sp, FALSE);
+	    SITEflush(sp, false);
 	return;
     }
 
@@ -879,7 +881,8 @@ void SITEflush(SITE *sp, BOOL Restart)
 /*
 **  Flush all sites.
 */
-void SITEflushall(BOOL Restart)
+void
+SITEflushall(const bool Restart)
 {
     int	                i;
     SITE	        *sp;
@@ -894,10 +897,11 @@ void SITEflushall(BOOL Restart)
 **  Run down the site's pattern list and see if it wants the specified
 **  newsgroup.
 */
-BOOL SITEwantsgroup(SITE *sp, char *name)
+bool
+SITEwantsgroup(SITE *sp, char *name)
 {
-    BOOL	        match;
-    BOOL	        subvalue;
+    bool	        match;
+    bool	        subvalue;
     char	        *pat;
     char	        **argv;
 
@@ -907,7 +911,7 @@ BOOL SITEwantsgroup(SITE *sp, char *name)
 	    subvalue = *pat != SUB_NEGATE && *pat != SUB_POISON;
 	    if (!subvalue)
 		pat++;
-	    if ((match != subvalue) && wildmat(name, pat))
+	    if ((match != subvalue) && uwildmat(name, pat))
 		match = subvalue;
 	}
     }
@@ -915,7 +919,7 @@ BOOL SITEwantsgroup(SITE *sp, char *name)
 	subvalue = *pat != SUB_NEGATE && *pat != SUB_POISON;
 	if (!subvalue)
 	    pat++;
-	if ((match != subvalue) && wildmat(name, pat))
+	if ((match != subvalue) && uwildmat(name, pat))
 	    match = subvalue;
     }
     return match;
@@ -926,10 +930,11 @@ BOOL SITEwantsgroup(SITE *sp, char *name)
 **  Run down the site's pattern list and see if specified newsgroup is
 **  considered poison.
 */
-BOOL SITEpoisongroup(SITE *sp, char *name)
+bool
+SITEpoisongroup(SITE *sp, char *name)
 {
-    BOOL	        match;
-    BOOL	        poisonvalue;
+    bool	        match;
+    bool	        poisonvalue;
     char	        *pat;
     char	        **argv;
 
@@ -939,7 +944,7 @@ BOOL SITEpoisongroup(SITE *sp, char *name)
 	    poisonvalue = *pat == SUB_POISON;
 	    if (*pat == SUB_NEGATE || *pat == SUB_POISON)
 		pat++;
-	    if (wildmat(name, pat))
+	    if (uwildmat(name, pat))
 		match = poisonvalue;
 	}
     }
@@ -947,7 +952,7 @@ BOOL SITEpoisongroup(SITE *sp, char *name)
 	poisonvalue = *pat == SUB_POISON;
 	if (*pat == SUB_NEGATE || *pat == SUB_POISON)
 	    pat++;
-	if (wildmat(name, pat))
+	if (uwildmat(name, pat))
 	    match = poisonvalue;
     }
     return match;
@@ -957,13 +962,14 @@ BOOL SITEpoisongroup(SITE *sp, char *name)
 /*
 **  Find a site.
 */
-SITE *SITEfind(char *p)
+SITE *
+SITEfind(const char *p)
 {
     int	                i;
     SITE	        *sp;
 
     for (i = nSites, sp = Sites; --i >= 0; sp++)
-	if (sp->Name && caseEQ(p, sp->Name))
+	if (sp->Name && strcasecmp(p, sp->Name) == 0)
 	    return sp;
     return NULL;
 }
@@ -972,12 +978,13 @@ SITE *SITEfind(char *p)
 /*
 **  Find the next site that matches this site.
 */
-SITE *SITEfindnext(char *p, SITE *sp)
+SITE *
+SITEfindnext(const char *p, SITE *sp)
 {
     SITE	        *end;
 
     for (sp++, end = &Sites[nSites]; sp < end; sp++)
-	if (sp->Name && caseEQ(p, sp->Name))
+	if (sp->Name && strcasecmp(p, sp->Name) == 0)
 	    return sp;
     return NULL;
 }
@@ -986,7 +993,8 @@ SITE *SITEfindnext(char *p, SITE *sp)
 /*
 **  Close a site down.
 */
-void SITEfree(SITE *sp)
+void
+SITEfree(SITE *sp)
 {
     SITE                *s;
     int                 new;
@@ -1006,56 +1014,57 @@ void SITEfree(SITE *sp)
 	sp->Process = -1;
     }
     if (sp->Entry) {
-	DISPOSE(sp->Entry);
+	free(sp->Entry);
 	sp->Entry = NULL;
     }
     if (sp->Originator) {
-    	DISPOSE(sp->Originator);
+    	free(sp->Originator);
     	sp->Originator = NULL;
     }
     if (sp->Param) {
-	DISPOSE(sp->Param);
+	free(sp->Param);
 	sp->Param = NULL;
     }
     if (sp->SpoolName) {
-	DISPOSE(sp->SpoolName);
+	free(sp->SpoolName);
 	sp->SpoolName = NULL;
     }
     if (sp->Patterns) {
-	DISPOSE(sp->Patterns);
+	free(sp->Patterns);
 	sp->Patterns = NULL;
     }
     if (sp->Exclusions) {
-	DISPOSE(sp->Exclusions);
+	free(sp->Exclusions);
 	sp->Exclusions = NULL;
     }
     if (sp->Distributions) {
-	DISPOSE(sp->Distributions);
+	free(sp->Distributions);
 	sp->Distributions = NULL;
     }
-    if (sp->Buffer.Data) {
-	DISPOSE(sp->Buffer.Data);
-	sp->Buffer.Data = NULL;
-	sp->Buffer.Size = 0;
+    if (sp->Buffer.data) {
+	free(sp->Buffer.data);
+	sp->Buffer.data = NULL;
+	sp->Buffer.size = 0;
     }
-    if (sp->FNLnames.Data) {
-	DISPOSE(sp->FNLnames.Data);
-	sp->FNLnames.Data = NULL;
-	sp->FNLnames.Size = 0;
+    if (sp->FNLnames.data) {
+	free(sp->FNLnames.data);
+	sp->FNLnames.data = NULL;
+	sp->FNLnames.size = 0;
     }
 
     /* If this site was a master, find a new one. */
     if (sp->IsMaster) {
 	for (new = NOSITE, s = Sites, i = nSites; --i >= 0; s++)
-	    if (&Sites[s->Master] == sp)
+	    if (&Sites[s->Master] == sp) {
 		if (new == NOSITE) {
 		    s->Master = NOSITE;
-		    s->IsMaster = TRUE;
+		    s->IsMaster = true;
 		    new = s - Sites;
 		}
 		else
 		    s->Master = new;
-	sp->IsMaster = FALSE;
+            }
+	sp->IsMaster = false;
     }
 }
 
@@ -1064,10 +1073,10 @@ void SITEfree(SITE *sp)
 **  If a site is an exploder or funnels into one, forward a command
 **  to it.
 */
-void SITEforward(SITE *sp, char *text)
+void
+SITEforward(SITE *sp, const char *text)
 {
     SITE	        *fsp;
-    char	        *p;
     char		buff[SMBUF];
 
     fsp = sp;
@@ -1076,11 +1085,10 @@ void SITEforward(SITE *sp, char *text)
     if (sp->Name == NULL || fsp->Name == NULL)
 	return;
     if (fsp->Type == FTexploder) {
-	(void)strcpy(buff, text);
+	strlcpy(buff, text, sizeof(buff));
 	if (fsp != sp && fsp->FNLwantsnames) {
-	    p = buff + strlen(buff);
-	    *p++ = ' ';
-	    (void)strcpy(p, sp->Name);
+            strlcat(buff, " ", sizeof(buff));
+            strlcat(buff, sp->Name, sizeof(buff));
 	}
 	SITEwrite(fsp, buff);
     }
@@ -1090,10 +1098,11 @@ void SITEforward(SITE *sp, char *text)
 /*
 **  Drop a site.
 */
-void SITEdrop(SITE *sp)
+void
+SITEdrop(SITE *sp)
 {
     SITEforward(sp, "drop");
-    SITEflush(sp, FALSE);
+    SITEflush(sp, false);
     SITEfree(sp);
 }
 
@@ -1101,45 +1110,48 @@ void SITEdrop(SITE *sp)
 /*
 **  Append info about the current state of the site to the buffer
 */
-void SITEinfo(BUFFER *bp, SITE *sp, BOOL Verbose)
+void
+SITEinfo(struct buffer *bp, SITE *sp, const bool Verbose)
 {
     static char		FREESITE[] = "<<No name>>\n\n";
     char	        *p;
     CHANNEL	        *cp;
-    char		*sep;
+    const char		*sep;
     char		buff[BUFSIZ];
 
     if (sp->Name == NULL) {
-	BUFFappend(bp, FREESITE, STRLEN(FREESITE));
+	buffer_append(bp, FREESITE, strlen(FREESITE));
 	return;
     }
 
     p = buff;
-    (void)sprintf(buff, "%s%s:\t", sp->Name, sp->IsMaster ? "(*)" : "");
+    snprintf(buff, sizeof(buff), "%s%s:\t", sp->Name,
+             sp->IsMaster ? "(*)" : "");
     p += strlen(p);
 
     if (sp->Type == FTfunnel) {
 	sp = &Sites[sp->Funnel];
-	(void)sprintf(p, "funnel -> %s: ", sp->Name);
+	sprintf(p, "funnel -> %s: ", sp->Name);
 	p += strlen(p);
     }
 
     switch (sp->Type) {
     default:
-	(void)sprintf(p, "unknown feed type %d", sp->Type);
+	sprintf(p, "unknown feed type %d", sp->Type);
 	p += strlen(p);
 	break;
     case FTerror:
     case FTfile:
 	p += strlen(strcpy(p, "file"));
 	if (sp->Buffered) {
-	    (void)sprintf(p, " buffered(%ld)", sp->Buffer.Left);
+	    sprintf(p, " buffered(%lu)", (unsigned long) sp->Buffer.left);
 	    p += strlen(p);
 	}
 	else if ((cp = sp->Channel) == NULL)
 	    p += strlen(strcpy(p, " no channel?"));
 	else {
-	    (void)sprintf(p, " open fd=%d, in mem %ld", cp->fd, cp->Out.Left);
+	    sprintf(p, " open fd=%d, in mem %lu", cp->fd,
+                    (unsigned long) cp->Out.left);
 	    p += strlen(p);
 	}
 	break;
@@ -1150,7 +1162,7 @@ void SITEinfo(BUFFER *bp, SITE *sp, BOOL Verbose)
 	p += strlen(strcpy(p, "exploder"));
 Common:
 	if (sp->Process >= 0) {
-	    (void)sprintf(p, " pid=%ld", (long) sp->pid);
+	    sprintf(p, " pid=%ld", (long) sp->pid);
 	    p += strlen(p);
 	}
 	if (sp->Spooling)
@@ -1158,7 +1170,8 @@ Common:
 	if ((cp = sp->Channel) == NULL)
 	    p += strlen(strcpy(p, " no channel?"));
 	else {
-	    (void)sprintf(p, " fd=%d, in mem %ld", cp->fd, cp->Out.Left);
+	    sprintf(p, " fd=%d, in mem %lu", cp->fd,
+                    (unsigned long) cp->Out.left);
 	    p += strlen(p);
 	}
 	break;
@@ -1178,41 +1191,41 @@ Common:
     if (Verbose) {
 	sep = "\t";
 	if (sp->Buffered && sp->Flushpoint) {
-	    (void)sprintf(p, "%sFlush @ %ld", sep, sp->Flushpoint);
+	    sprintf(p, "%sFlush @ %ld", sep, sp->Flushpoint);
 	    p += strlen(p);
 	    sep = "; ";
 	}
 	if (sp->StartWriting || sp->StopWriting) {
-	    (void)sprintf(p, "%sWrite [%ld..%ld]", sep,
+	    sprintf(p, "%sWrite [%ld..%ld]", sep,
 		sp->StopWriting, sp->StartWriting);
 	    p += strlen(p);
 	    sep = "; ";
 	}
 	if (sp->StartSpooling) {
-	    (void)sprintf(p, "%sSpool @ %ld", sep, sp->StartSpooling);
+	    sprintf(p, "%sSpool @ %ld", sep, sp->StartSpooling);
 	    p += strlen(p);
 	    sep = "; ";
 	}
 	if (sep[0] != '\t')
 	    *p++ = '\n';
 	if (sp->Spooling && sp->SpoolName) {
-	    (void)sprintf(p, "\tSpooling to \"%s\"\n", sp->SpoolName);
+	    sprintf(p, "\tSpooling to \"%s\"\n", sp->SpoolName);
 	    p += strlen(p);
 	}
 	if ((cp = sp->Channel) != NULL) {
-	    (void)sprintf(p, "\tChannel created %.12s",
+	    sprintf(p, "\tChannel created %.12s",
 		ctime(&cp->Started) + 4);
 	    p += strlen(p);
-	    (void)sprintf(p, ", last active %.12s\n",
+	    sprintf(p, ", last active %.12s\n",
 		ctime(&cp->LastActive) + 4);
 	    p += strlen(p);
 	    if (cp->Waketime > Now.time) {
-		(void)sprintf(p, "\tSleeping until %.12s\n",
+		sprintf(p, "\tSleeping until %.12s\n",
 		    ctime(&cp->Waketime) + 4);
 		p += strlen(p);
 	    }
 	}
 
     }
-    BUFFappend(bp, buff, p - buff);
+    buffer_append(bp, buff, p - buff);
 }
