@@ -1,72 +1,40 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Wed Dec 27 18:45:55 1995
- * Project:     INN (innfeed)
- * File:        article.c
- * RCSId:       $Id: article.c,v 1.15 1998/04/28 12:04:28 scrappy Exp $
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- * Description: The implementation of the Article class. Articles are the
- *              abstraction for the actual news articles. They are a
- *              reference counted object because they need to be shared by
- *              multiple Host and Connection objects. When an Article is
- *              created it verifies that the actual file exists. When a
- *              Connection wants the article's contents for transmission
- *              the Article reads the data off disk and returns a set of
- *              Buffer objects. The Article holds onto these Buffers so
- *              that the next Connection that wants to transmit it won't
- *              have to wait for a disk read to be done again.
- *              
- * 
- */
-
-#if ! defined (lint)
-static const char *rcsid = "$Id: article.c,v 1.15 1998/04/28 12:04:28 scrappy Exp $" ;
-static void use_rcsid (const char *rid) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rid) ;
-}
-#endif
+/*  $Id: article.c,v 1.33.2.2 2004/05/16 20:26:54 rra Exp $
+**
+**  The Article class for innfeed.
+**
+**  Written by James Brister <brister@vix.com>
+**
+**  The implementation of the Article class. Articles are the abstraction for
+**  the actual news articles. They are a reference counted object because they
+**  need to be shared by multiple Host and Connection objects. When an Article
+**  is created it verifies that the actual file exists. When a Connection
+**  wants the article's contents for transmission the Article reads the data
+**  off disk and returns a set of Buffer objects. The Article holds onto these
+**  Buffers so that the next Connection that wants to transmit it won't have
+**  to wait for a disk read to be done again.
+*/
 
+#include "innfeed.h"
 #include "config.h"
+#include "clibrary.h"
+#include "portable/mmap.h"
 
-#if defined (HAVE_UNISTD_H)
-# include <unistd.h>
-#endif
-
-#include <stdlib.h>
 #include <assert.h>
-#include <sys/types.h>
-#include <sys/mman.h>
+#include <errno.h>
+#include <fcntl.h>
+#if HAVE_LIMITS_H
+# include <limits.h>
+#endif
 #include <sys/stat.h>
 #include <syslog.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <string.h>
-#include <limits.h>
 
-#include "configdata.h"
-#include "clibrary.h"
+#include "inn/messages.h"
 #include "libinn.h"
+#include "storage.h"
+
 #include "article.h"
 #include "buffer.h"
 #include "endpoint.h"
-#include "msgs.h"
 
 #if defined (NDEBUG)
 #define VALIDATE_HASH_TABLE() (void (0))
@@ -83,7 +51,7 @@ struct article_s
     char *msgid ;               /* the msgid of the article (INN tells us) */
     Buffer contents ;           /* the buffer of the actual on disk stuff */
     Buffer *nntpBuffers ;       /* list of buffers for transmisson */
-    caddr_t mMapping ;          /* base of memory mapping, or NULL if none */
+    const void *mMapping ;      /* base of memory mapping, or NULL if none */
     bool loggedMissing ;        /* true if article is missing and we logged */
     bool articleOk ;            /* true until we know otherwise. */
     bool inWireFormat ;         /* true if ->contents is \r\n/dot-escaped */
@@ -95,7 +63,7 @@ struct hash_entry_s {
     struct hash_entry_s *prev ;
     struct hash_entry_s *nextTime ;
     struct hash_entry_s *prevTime ;
-    u_int hash ;
+    unsigned int hash ;
     Article article ;
 };
 
@@ -125,7 +93,8 @@ static bool artFreeContents (Article art) ;  /* Tell the Article to release
                                                 its contents buffer if
                                                 possible. */
 
-static void artUnmap (Article art, int size) ; /* munmap an mmap()ed article */
+static void artUnmap (Article art, size_t size) ; /* munmap an mmap()ed
+                                                     article */
 
 
   /*
@@ -133,7 +102,7 @@ static void artUnmap (Article art, int size) ; /* munmap an mmap()ed article */
    */
 
   /* Returns a has value for the given string */
-static u_int hashString (const char *string) ;
+static unsigned int hashString (const char *string) ;
 
   /* Locates the article with the given message ID, in the has table. */
 static Article hashFindArticle (const char *msgid) ;
@@ -154,36 +123,36 @@ static void hashValidateTable (void) ;
    */
 
 
-static u_int missingArticleCount ;  /* Number of articles that were missing */
+static unsigned int missingArticleCount ;  /* Number of articles that were missing */
 
 static bool logMissingArticles ;  /* if true then we log the details on a
                                      missing article. */ 
 
-static u_int preparedBytes ;  /* The number of areticle bytes read in so
+static unsigned int preparedBytes ;  /* The number of areticle bytes read in so
                                  far of disk (will wrap around) */
 
-static u_int preparedNewlines ; /* The number of newlines found in all the
+static unsigned int preparedNewlines ; /* The number of newlines found in all the
                                    preparedBytes */
 
-static u_int avgCharsPerLine ;  /* The average number of characters per
+static unsigned int avgCharsPerLine ;  /* The average number of characters per
                                    line over all articles. */
 
 static bool rolledOver ;  /* true if preparedBytes wrapped around */
 
-static u_int bytesInUse ;   /* count of article contents bytes in use--just
+static unsigned int bytesInUse ;   /* count of article contents bytes in use--just
                                the amount read from the article files, not
                                all memory involved in. */
 
-static u_int maxBytesInUse ;   /* the limit we want to stay under for total
+static unsigned int maxBytesInUse ;   /* the limit we want to stay under for total
                                   bytes resident in memory dedicated to
                                   article contents. */
 
-static u_int articlesInUse ;  /* number of articles currently allocated. */
+static unsigned int articlesInUse ;  /* number of articles currently allocated. */
 
-static u_int byteTotal ;        /* number of bytes for article contents
+static unsigned int byteTotal ;        /* number of bytes for article contents
                                    allocated totally since last log. */
 
-static u_int articleTotal ; /* number of articles alloced since last log. */
+static unsigned int articleTotal ; /* number of articles alloced since last log. */
 
 static TimeoutId articleStatsId ; /* The timer callback id. */
 
@@ -213,13 +182,13 @@ Article newArticle (const char *filename, const char *msgid)
 {
   Article newArt = NULL ;
   
+  TMRstart(TMR_NEWARTICLE);
   if (hashTable == NULL)
     {                           /* first-time through initialization. */
       int i ;
       
       ASSERT ((TABLE_SIZE & HASH_MASK) == 0) ;
-      hashTable = ALLOC (HashEntry, TABLE_SIZE) ;
-      ASSERT (hashTable != NULL) ;
+      hashTable = xmalloc (sizeof(HashEntry) * TABLE_SIZE) ;
       
       addPointerFreedOnExit ((char *)hashTable) ;
 
@@ -234,16 +203,10 @@ Article newArticle (const char *filename, const char *msgid)
        ok */
   if ((newArt = hashFindArticle (msgid)) == NULL)
     {
-      newArt = CALLOC (struct article_s, 1) ;
-      ASSERT (newArt != NULL) ;
-      
-      newArt->fname = MALLOC (strlen (filename) + 1) ;
-      ASSERT (newArt->fname != NULL) ;
-      strcpy (newArt->fname,filename) ;
-      
-      newArt->msgid = MALLOC (strlen (msgid) + 1) ;
-      ASSERT (newArt->msgid != NULL) ;
-      strcpy (newArt->msgid,msgid) ;
+      newArt = xcalloc (1, sizeof(struct article_s)) ;
+
+      newArt->fname = xstrdup (filename) ;
+      newArt->msgid = xstrdup (msgid) ;
       
       newArt->contents = NULL ;
       newArt->mMapping = NULL ;
@@ -253,7 +216,7 @@ Article newArticle (const char *filename, const char *msgid)
       newArt->inWireFormat = false ;
       newArt->arthandle = NULL;
       
-      d_printf (3,"Adding a new article(%p): %s\n", newArt, msgid) ;
+      d_printf (3,"Adding a new article(%p): %s\n", (void *)newArt, msgid) ;
       
       articlesInUse++ ;
       articleTotal++ ;
@@ -263,12 +226,13 @@ Article newArticle (const char *filename, const char *msgid)
   else
     {
       if (strcmp (filename,newArt->fname) != 0)
-        syslog (LOG_ERR, DOUBLE_NAME, filename, newArt->fname) ;
+        warn ("ME two filenames for same article: %s, %s", filename,
+              newArt->fname) ;
       
       newArt->refCount++ ;
       d_printf (2,"Reusing existing article for %s\nx",msgid) ;
     }
-  
+  TMRstop(TMR_NEWARTICLE);
   return newArt ;
 }
 
@@ -288,7 +252,8 @@ void delArticle (Article article)
 
       ASSERT (removed == true) ;
 
-      d_printf (2,"Cleaning up article (%p): %s\n",article, article->msgid) ;
+      d_printf (2,"Cleaning up article (%p): %s\n", 
+          (void *)article, article->msgid) ;
 
       if (article->contents != NULL)
         {
@@ -304,20 +269,20 @@ void delArticle (Article article)
         }
 
       articlesInUse-- ;
-      
-      FREE (article->fname) ;
-      FREE (article->msgid) ;
-      FREE (article) ;
+
+      free (article->fname) ;
+      free (article->msgid) ;
+      free (article) ;
     }
 
   VALIDATE_HASH_TABLE () ;
 }
 
 
-void gPrintArticleInfo (FILE *fp, u_int indentAmt)
+void gPrintArticleInfo (FILE *fp, unsigned int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
-  u_int i ;
+  unsigned int i ;
 
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
@@ -350,17 +315,17 @@ void gPrintArticleInfo (FILE *fp, u_int indentAmt)
 }
 
 
-void printArticleInfo (Article art, FILE *fp, u_int indentAmt)
+void printArticleInfo (Article art, FILE *fp, unsigned int indentAmt)
 {
   Buffer *b ;
   char indent [INDENT_BUFFER_SIZE] ;
-  u_int i ;
+  unsigned int i ;
 
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
   
-  fprintf (fp,"%sArticle : %p {\n",indent,art) ;
+  fprintf (fp,"%sArticle : %p {\n",indent,(void *) art) ;
   fprintf (fp,"%s    article ok : %s\n",indent,boolToString (art->articleOk)) ;
   fprintf (fp,"%s    refcount : %d\n",indent,art->refCount) ;
   fprintf (fp,"%s    filename : %s\n",indent,art->fname) ;
@@ -440,9 +405,17 @@ const char *artMsgId (Article article)
   return article->msgid ;
 }
 
+  /* return size of the article */
+int artSize (Article article)
+{
+  if (article == NULL || article->contents == NULL)
+    return (int)0 ;
+  return (int)bufferDataSize(article->contents);
+}
+
 
   /* return how many NNTP-ready buffers the article contains */
-u_int artNntpBufferCount (Article article)
+unsigned int artNntpBufferCount (Article article)
 {
   if ( !prepareArticleForNNTP (article) )
     return 0 ;
@@ -459,7 +432,7 @@ void artLogMissingArticles (bool val)
 
 
   /* set the limit we want to stay under. */
-void artSetMaxBytesInUse (u_int val)
+void artSetMaxBytesInUse (unsigned int val)
 {
   ASSERT (maxBytesInUse > 0) ;  /* can only set one time. */
   ASSERT (val > 0) ;
@@ -492,7 +465,7 @@ static Buffer artGetContents (Article article)
 }
 
 
-static void artUnmap (Article article, int size) {
+static void artUnmap (Article article, size_t size) {
     
     if (article->arthandle)
 	SMfreearticle(article->arthandle);
@@ -506,14 +479,12 @@ static void artUnmap (Article article, int size) {
 
 
 
-static void logArticleStats (TimeoutId id, void *data)
+static void logArticleStats (TimeoutId id, void *data UNUSED)
 {
   ASSERT (id == articleStatsId) ;
 
-  (void) data ;                 /* keep lint happy */
-
-  syslog (LOG_NOTICE,ACTIVE_ARTICLES,articlesInUse,bytesInUse) ;
-  syslog (LOG_NOTICE,ARTICLE_ALLOCS,articleTotal,byteTotal) ;
+  notice ("ME articles active %d bytes %d", articlesInUse, bytesInUse) ;
+  notice ("ME articles total %d bytes %d", articleTotal, byteTotal) ;
   
   byteTotal = 0 ;
   articleTotal = 0 ;
@@ -532,11 +503,11 @@ static void logArticleStats (TimeoutId id, void *data)
 
 static bool fillContents (Article article)
 {
-    int fd ;
+    int fd = -1;
     char *p;
     static bool maxLimitNotified ;
     bool opened;
-    int articlesize;
+    size_t articlesize = 0;
     char *buffer = NULL ;
     int amt = 0 ;
     size_t idx = 0, amtToRead ;
@@ -545,6 +516,7 @@ static bool fillContents (Article article)
   
     ASSERT (article->contents == NULL) ;
     
+    TMRstart(TMR_READART);
     if (maxBytesInUse == 0)
 	maxBytesInUse = SOFT_ARTICLE_BYTE_LIMIT ;
     
@@ -560,6 +532,7 @@ static bool fillContents (Article article)
 		syslog(LOG_ERR, "Could not retrieve %s: %s",
 		       article->fname, SMerrorstr);
 		article->articleOk = false;
+		TMRstop(TMR_READART); 
 		return false;
 	    }
 	}
@@ -571,17 +544,20 @@ static bool fillContents (Article article)
 	if (opened) {
 	    if (fstat (fd, &sb) < 0) {
 		article->articleOk = false ;
-		syslog (LOG_ERR,FSTAT_FAILURE,article->fname) ;
+                syswarn ("ME oserr fstat %s", article->fname) ;
+		TMRstop(TMR_READART);
 		return false;
 	    }
 	    if (!S_ISREG (sb.st_mode)) {
 		article->articleOk = false ;
-		syslog (LOG_ERR,REGFILE_FAILURE,article->fname) ;
+                warn ("ME article file-type: %s", article->fname) ;
+		TMRstop(TMR_READART);
 		return false;
 	    }
 	    if (sb.st_size == 0) {
 		article->articleOk = false ;
-		syslog (LOG_ERR,EMPTY_ARTICLE,article->fname) ;
+                warn ("ME article 0 bytes: %s", article->fname) ;
+		TMRstop(TMR_READART);
 		return false;
 	    }
 	    articlesize = sb.st_size;
@@ -594,35 +570,39 @@ static bool fillContents (Article article)
 	
 	if (logMissingArticles && !article->loggedMissing)
 	{
-	    syslog (LOG_NOTICE,NO_ARTICLE,article->msgid,article->fname) ;
+            notice ("ME article missing: %s, %s", article->msgid,
+                    article->fname) ;
 	    article->loggedMissing = true ;
 	}
+	TMRstop(TMR_READART);
 	return false;
     }
-    amtToRead = (size_t) articlesize ;
-    newBufferSize = (size_t) articlesize ;
+    amtToRead = articlesize ;
+    newBufferSize = articlesize ;
     
     if (article->arthandle || useMMap) {
 	if (article->arthandle)
 	    article->mMapping = article->arthandle->data;
 	else 
-	    article->mMapping = mmap((caddr_t) 0, (size_t) articlesize,
-				     PROT_READ, MAP_SHARED, fd, 0);
+	    article->mMapping = mmap(NULL, articlesize, PROT_READ,
+                                     MAP_SHARED, fd, 0);
 	
-	if (article->mMapping == (caddr_t) -1) {
+	if (article->mMapping == MAP_FAILED) {
 	    /* dunno, but revert to plain reading */
 	    article->mMapping = NULL ;
-	    syslog (LOG_NOTICE, MMAP_FAILURE, article->fname) ;
+            syswarn ("ME mmap failure %s (%s)",
+                     article->fname,
+                     strerror(errno)) ;
 	} else {
-	    article->contents = newBufferByCharP((char *)article->mMapping,
-						 (size_t) articlesize,
-						 (size_t) articlesize);
+	    article->contents = newBufferByCharP(article->mMapping,
+						 articlesize,
+						 articlesize);
 	    buffer = bufferBase (article->contents) ;
 	    if ((p = strchr(buffer, '\n')) == NULL) {
 		article->articleOk = false;
 		delBuffer (article->contents) ;
 		article->contents = NULL ;
-		syslog (LOG_NOTICE, MUNGED_ARTICLE, article->fname) ;
+                warn ("ME munged article %s", article->fname) ;
 	    } else {
 		if (p[-1] == '\r') {
 		    article->inWireFormat = true ;
@@ -655,8 +635,9 @@ static bool fillContents (Article article)
 	/* we we couldn't get below, then log it (one time only) */
 	if ((amtToRead + bytesInUse) > maxBytesInUse && maxLimitNotified == false) {
 	    maxLimitNotified = true ;
-	    syslog (LOG_NOTICE,MAX_BYTES_LIMIT,maxBytesInUse,
-		    amtToRead + bytesInUse) ;
+            notice ("ME exceeding maximum article byte limit: %d (max),"
+                    " %lu (cur)", maxBytesInUse, 
+                    (unsigned long) (amtToRead + bytesInUse)) ;
 	}
 	
 	if ((article->contents = newBuffer (newBufferSize)) == NULL)
@@ -668,15 +649,14 @@ static bool fillContents (Article article)
 	}
 	
 	if (article->mMapping && buffer != NULL) {               
-	    (void)memcpy(buffer, (char *) article->mMapping,
-			 (int) articlesize);
-	    artUnmap(article, (int) articlesize) ;
+	    memcpy(buffer, article->mMapping, articlesize);
+	    artUnmap(article, articlesize) ;
 	    amtToRead = 0;
 	}
 	
 	while (amtToRead > 0) {
 	    if ((amt = read (fd, buffer + idx,amtToRead)) <= 0) {
-		syslog (LOG_ERR,BAD_ART_READ, article->fname) ;
+                syswarn ("ME article read error: %s", article->fname) ;
 		bytesInUse -= articlesize ;
 		byteTotal -= articlesize ;
 		amtToRead = 0 ;
@@ -691,11 +671,11 @@ static bool fillContents (Article article)
 	}
 	
 	if (article->contents != NULL) {
-	    bufferSetDataSize (article->contents, (size_t) articlesize) ;
+	    bufferSetDataSize (article->contents, articlesize) ;
 	    
 	    if ((p = strchr(buffer, '\n')) == NULL) {                  
 		article->articleOk = false;
-		syslog (LOG_NOTICE, MUNGED_ARTICLE, article->fname) ;
+                warn ("ME munged article %s", article->fname) ;
 	    }
 	    else if (p[-1] == '\r') {
 		article->inWireFormat = true ;
@@ -705,10 +685,10 @@ static bool fillContents (Article article)
 		    size_t diff =
 			(bufferDataSize (article->contents) - articlesize) ;
 		    
-		    if (((u_int) UINT_MAX) - diff <= preparedBytes) {
+		    if (((unsigned int) UINT_MAX) - diff <= preparedBytes) {
 			d_printf (2,"Newline ratio so far: %02.2f\n",
 				 ((double) preparedBytes / preparedNewlines)) ;
-			syslog (LOG_NOTICE,PREPARED_NEWLINES,
+                        notice ("ME newline to file size ratio: %0.2f (%d/%d)",
 				((double) preparedBytes)/preparedNewlines,
 				preparedBytes,preparedNewlines) ;
 			preparedBytes = 0 ;
@@ -728,7 +708,7 @@ static bool fillContents (Article article)
 		    }
 		    article->inWireFormat = true ;
 		} else {
-		    syslog (LOG_ERR,PREPARE_FAILED) ;
+                    warn ("ME internal failed to prepare buffer for NNTP") ;
 		    bytesInUse -= articlesize ;
 		    byteTotal -= articlesize ;
 		    
@@ -743,7 +723,8 @@ static bool fillContents (Article article)
     /* If we're not useing storage api, we should close a valid file descriptor */
     if (!article->arthandle && (fd >= 0))
 	close (fd) ;
-  
+
+    TMRstop(TMR_READART);
     return (article->contents != NULL ? true : false) ;
 }
 
@@ -758,8 +739,7 @@ static void appendBuffer (Buffer b, Buffer **buffs, int *newSpot, int *curLen)
   if (*newSpot == *curLen)
     {
       *curLen += 10 ;
-      *buffs = REALLOC (*buffs, Buffer, *curLen) ;
-      ASSERT (*buffs != NULL) ;
+      *buffs = xrealloc (*buffs, sizeof(Buffer) * *curLen) ;
     }
   (*buffs) [(*newSpot)++] = b ;
 }
@@ -778,13 +758,19 @@ static bool prepareArticleForNNTP (Article article)
   int buffLen = 0 ;
   int buffIdx = 0 ;
   char *start, *end ;
-  Buffer contents = artGetContents (article) ; /* returns a reference */
+  Buffer contents ;
 
-  if (contents == NULL)
+  contents = artGetContents (article) ; /* returns a reference */
+
+  TMRstart(TMR_PREPART);
+  if (contents == NULL) {
+    TMRstop(TMR_PREPART);
     return false ;
+  }
   else if (article->nntpBuffers != NULL)
     {
       delBuffer (contents) ;
+      TMRstop(TMR_PREPART);
       return true ;               /* already done */
     }
 
@@ -823,9 +809,8 @@ static bool prepareArticleForNNTP (Article article)
     }
   else
     {
-        /* we already fixed the contents up when we read in the article */
-      nntpBuffs = ALLOC (Buffer, 3) ;
-      ASSERT (nntpBuffs != NULL) ;
+      /* we already fixed the contents up when we read in the article */
+      nntpBuffs = xmalloc (sizeof(Buffer) * 3) ;
       nntpBuffs [0] = newBufferByCharP (bufferBase (contents),
                                         bufferDataSize (contents),
                                         bufferDataSize (contents)) ;
@@ -835,7 +820,7 @@ static bool prepareArticleForNNTP (Article article)
       
   delBuffer (contents) ;    /* the article is still holding a reference */
   article->nntpBuffers = nntpBuffs ;
-
+  TMRstop(TMR_PREPART);
   return true ;
 }
 
@@ -848,6 +833,7 @@ static bool artFreeContents (Article art)
     return false ;
 
   if (art->nntpBuffers != NULL)
+    {
     if (bufferRefCount (art->nntpBuffers[0]) > 1)
       return false ;
     else
@@ -855,6 +841,7 @@ static bool artFreeContents (Article art)
         freeBufferArray (art->nntpBuffers) ;    
         art->nntpBuffers = NULL ;
       }
+    }
 
   ASSERT (bufferRefCount (art->contents) == 1) ;
 
@@ -885,9 +872,9 @@ static bool artFreeContents (Article art)
 
 
   /* Hash function lifted from perl 5 */
-static u_int hashString (const char *string)
+static unsigned int hashString (const char *string)
 {
-  u_int i ;
+  unsigned int i ;
 
   for (i = 0 ; string && *string ; string++)
     i = 33 * i + (u_char) *string ;
@@ -899,7 +886,7 @@ static u_int hashString (const char *string)
   /* find the article in the has table and return it. */
 static Article hashFindArticle (const char *msgid)
 {
-  u_int hash = hashString (msgid) ;
+  unsigned int hash = hashString (msgid) ;
   HashEntry h ;
 
   for (h = hashTable [TABLE_ENTRY(hash)] ; h != NULL ; h = h->next)
@@ -913,14 +900,13 @@ static Article hashFindArticle (const char *msgid)
   /* add the article to the hash table. */
 static void hashAddArticle (Article article)
 {
-  u_int hash = hashString (article->msgid) ;
+  unsigned int hash = hashString (article->msgid) ;
   HashEntry h ;
   HashEntry ne ;
 
   h = hashTable [TABLE_ENTRY(hash)] ;
 
-  ne = ALLOC (struct hash_entry_s, 1) ;
-  ASSERT (ne != NULL) ;
+  ne = xmalloc (sizeof(struct hash_entry_s));
 
   ne->article = article ;
   ne->hash = hash ;
@@ -946,7 +932,7 @@ static void hashAddArticle (Article article)
      Does not delete the article itself. */
 static bool hashRemoveArticle (Article article)
 {
-  u_int hash = hashString (article->msgid) ;
+  unsigned int hash = hashString (article->msgid) ;
   HashEntry h ;
 
   for (h = hashTable [TABLE_ENTRY(hash)] ; h != NULL ; h = h->next)
@@ -981,8 +967,8 @@ static bool hashRemoveArticle (Article article)
       if (h->nextTime != NULL)
         h->nextTime->prevTime = h->prevTime ;
     }
-  
-  FREE (h) ;
+
+  free (h) ;
   
   return true ;
 }
