@@ -1,113 +1,52 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Wed Nov 29 23:08:24 1995
- * Project:     INN (innfeed)
- * File:        endpoint.c
- * RCSId:       $Id: endpoint.c,v 1.14 1998/07/11 03:47:57 kondou Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: The implementation of the EndPoint object class.
- *
- *              The EndPoint class is what gives the illusion 9sort
- *              of, kind of) of threading. Basically it controls a
- *              select loop and a set of EndPoint objects. Each
- *              EndPoint has a file descriptor it is interested
- *              in. The users of the EndPoint tell the EndPoints to
- *              notify them when a read or write has been completed
- *              (or simple if the file descriptor is read or write
- *              ready).
- *
- */
-
-#if ! defined (lint)
-static char rcsid [] = "$Id: endpoint.c,v 1.14 1998/07/11 03:47:57 kondou Exp $" ;
-static void use_rcsid (const char *rId) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rId) ;
-} 
-#endif
+/*  $Id: endpoint.c,v 1.31.2.1 2004/01/25 20:07:10 rra Exp $
+**
+**  The implementation of the innfeed EndPoint object class.
+**
+**  Written by James Brister <brister@vix.com>
+**
+**  The EndPoint class is what gives the illusion (sort of, kind of) of
+**  threading.  Basically it controls a select loop and a set of EndPoint
+**  objects.  Each EndPoint has a file descriptor it is interested in.  The
+**  users of the EndPoint tell the EndPoints to notify them when a read or
+**  write has been completed (or simple if the file descriptor is read or
+**  write ready).
+*/
 
+#include "innfeed.h"
 #include "config.h"
+#include "clibrary.h"
+#include "portable/socket.h"
+#include "portable/time.h"
 
-
-#if defined (HAVE_UNISTD_H)
-# include <unistd.h>
-#endif /* defined (HAVE_UNISTD_H) */
-
-#include <stdlib.h>
 #include <assert.h>
-#include <string.h>
-#include <stdio.h>
 #include <errno.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
-
-#if defined (DO_NEED_SYS_SELECT) 
-#include <sys/select.h>
-#endif
-
-#if defined (DO_NEED_TIME)
-#include <time.h>
-#endif /* defined (DO_NEED_TIME) */
-#include <sys/time.h>
-
-#include <syslog.h>
+#include <signal.h>
+#include <sys/stat.h>
 #include <sys/uio.h>
-#include <sys/socket.h>
+#include <syslog.h>
 
-#if defined (DO_NEED_STREAM)
-#include <sys/stream.h>
+#ifdef HAVE_LIMITS_H
+# include <limits.h>
 #endif
 
+#ifdef HAVE_SYS_SELECT_H
+# include <sys/select.h>
+#endif
 
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "libinn.h"
+
+#include "buffer.h"
 #include "configfile.h"
 #include "endpoint.h"
-#include "buffer.h"
-#include "msgs.h"
 #include "host.h"
-#include "clibrary.h"
 
-
-#if defined (__bsdi__) && (defined (_ANSI_SOURCE) || defined (_POSIX_SOURCE))
-  /* why do I have to do this???? */
-struct timeval
-{
-    long tv_sec;                /* seconds */
-    long tv_usec;               /* and microseconds */
+static const char *const timer_name[] = {
+  "idle", "blstats", "stsfile", "newart", "readart", "prepart", "read",
+  "write", "cb"
 };
-#endif
-
-
-#if ! defined (UIO_MAXIOV) && defined (MAX_WRITEV_VEC)
-#define UIO_MAXIOV MAX_WRITEV_VEC
-#elif ! defined (UIO_MAXIOV) && defined (DEF_MAX_IOV) /* svr4 */
-#define UIO_MAXIOV DEF_MAX_IOV
-#endif
-
-
-#if ! defined (UIO_MAXIOV)
-#error Neither UIO_MAXIOV nor MAX_WRITEV_VEC are defined -- see sysconfig.h
-#endif
-
 
 #if ! defined (NSIG)
 #define NSIG 32
@@ -119,7 +58,7 @@ struct endpoint_s
 {
       /* fields for managing multiple reads into the inBuffer. */
     Buffer *inBuffer ;          /* list of buffers to read into */
-    u_int inBufferIdx ;         /* where is list we're at. */
+    unsigned int inBufferIdx ;         /* where is list we're at. */
     size_t inIndex ;            /* where in current read we're at */
     size_t inMinLen ;           /* minimum amount to read */
     size_t inAmtRead ;          /* amount read so far */
@@ -128,7 +67,7 @@ struct endpoint_s
     
       /* fields for managing multiple writes from the outBuffer */
     Buffer *outBuffer ;         /* list of buffers to write */
-    u_int outBufferIdx ;        /* index into buffer list to start write */
+    unsigned int outBufferIdx ;        /* index into buffer list to start write */
     size_t outIndex ;           /* where in current buffer we write from */
     size_t outSize ;            /* total of all the buffers */
     size_t outAmtWritten ;      /* amount written so far */
@@ -167,7 +106,7 @@ extern const char *InputFile ;
 
 static EndPoint mainEndPoint ;
 static bool mainEpIsReg = false ;
-static u_int stdioFdMax = MAX_STDIO_FD ;
+static unsigned int stdioFdMax = MAX_STDIO_FD ;
 
 time_t  PrivateTime;
 
@@ -175,8 +114,7 @@ time_t  PrivateTime;
 typedef void (*sigfn) (int) ;
 static sigfn *sigHandlers ;
 
-/* XXX - should be sig_atomic_t if we have it */
-static VOLATILE int *sigFlags ;
+static volatile sig_atomic_t *sigFlags ;
 
 
 
@@ -195,8 +133,8 @@ static void doTimeout (void) ;
 static void handleSignals (void) ;
 
 #if 0
-static int ff_set (fd_set *set, u_int start) ;
-static int ff_free (fd_set *set, u_int start) ;
+static int ff_set (fd_set *set, unsigned int start) ;
+static int ff_free (fd_set *set, unsigned int start) ;
 #endif
 static void endpointCleanup (void) ;
 
@@ -209,8 +147,8 @@ static EndPoint *priorityList ; /* endpoints indexed on priority */
 
 static int absHighestFd = 0 ;       /* never goes down */
 static int highestFd = -1 ;
-static u_int endPointCount = 0 ;
-static u_int priorityCount = 0 ;
+static unsigned int endPointCount = 0 ;
+static unsigned int priorityCount = 0 ;
 
 static fd_set rdSet ;
 static fd_set wrSet ;
@@ -247,14 +185,14 @@ EndPoint newEndPoint (int fd)
 
   /* try to dup the fd to a larger number to leave lower values free for
      broken stdio implementations. */
-  if (stdioFdMax > 0 && ((u_int) fd) <= stdioFdMax)
+  if (stdioFdMax > 0 && ((unsigned int) fd) <= stdioFdMax)
     {
       int newfd = fcntl(fd, F_DUPFD, stdioFdMax + 1);
       if (newfd >= 0)
         {
           d_printf (1,"Dupped fd %d to %d\n",fd,newfd) ;
           if (close (fd) != 0)
-            syslog (LOG_ERR,CLOSE_FAILED,fd) ;
+            syswarn ("ME oserr close (%d)", fd) ;
         }
       else
         {
@@ -265,20 +203,21 @@ EndPoint newEndPoint (int fd)
       fd = newfd ;
     }
 
-  if ((u_int) fd >= maxEndPoints)
+  if ((unsigned int) fd >= maxEndPoints)
     {
-      u_int i = maxEndPoints ;
+      unsigned int i = maxEndPoints ;
       
       maxEndPoints = (((fd + 256) / 256) * 256); /* round up to nearest 256 */ 
       if (endPoints == NULL)
         {
-          endPoints = ALLOC (EndPoint,maxEndPoints) ;
-          priorityList = ALLOC (EndPoint,maxEndPoints) ;
+          endPoints = xmalloc (sizeof(EndPoint) * maxEndPoints) ;
+          priorityList = xmalloc (sizeof(EndPoint) * maxEndPoints) ;
         }
       else
         {
-          endPoints = REALLOC (endPoints,EndPoint,maxEndPoints) ;
-          priorityList = REALLOC (priorityList,EndPoint,maxEndPoints) ;
+          endPoints = xrealloc (endPoints,sizeof(EndPoint) * maxEndPoints) ;
+          priorityList = xrealloc (priorityList,
+                                   sizeof(EndPoint) * maxEndPoints) ;
         }
 
       for ( ; i < maxEndPoints ; i++)
@@ -295,15 +234,16 @@ EndPoint newEndPoint (int fd)
       if (fd >= FD_SETSIZE)
         {
           sizelogged = true ;
-          syslog (LOG_ERR,FD_TOO_BIG,fd,FD_SETSIZE,"FD_SETSIZE") ;
+          warn ("ME fd (%d) looks too big (%d -- FD_SETSIZE)", fd,
+                FD_SETSIZE) ;
           return NULL ;
         }
 #else
       if (fd > (sizeof (fd_set) * CHAR_BIT))
         {
           sizelogged = true ;
-          syslog (LOG_ERR,FD_TOO_BIG,fd,(sizeof (fd_set) * CHAR_BIT),
-                  "(sizeof (fd_set) * CHAR_BIT)");
+          warn ("ME fd (%d) looks too big (%d -- sizeof (fd_set) * CHAR_BIT)",
+                fd, (sizeof (fd_set) * CHAR_BIT)) ;
           return NULL ;
         }
 #endif
@@ -311,8 +251,7 @@ EndPoint newEndPoint (int fd)
       absHighestFd = fd ;
     }
       
-  ep = CALLOC (struct endpoint_s, 1) ;
-  ASSERT (ep != NULL) ;
+  ep = xcalloc (1, sizeof(struct endpoint_s)) ;
 
   ep->inBuffer = NULL ;
   ep->inBufferIdx = 0 ;
@@ -355,7 +294,7 @@ EndPoint newEndPoint (int fd)
 
 void delEndPoint (EndPoint ep) 
 {
-  u_int idx ;
+  unsigned int idx ;
 
   if (ep == NULL)
     return ;
@@ -396,7 +335,7 @@ void delEndPoint (EndPoint ep)
 
   endPointCount-- ;
 
-  FREE (ep) ;
+  free (ep) ;
 }
 
 int endPointFd (EndPoint endp)
@@ -532,11 +471,8 @@ void cancelRead (EndPoint endp)
   copied to buffer. The number of bytes copied (if it is less than *len) is
   copied to len. If no write was outstanding then len will have 0 stored in
   it. */
-void cancelWrite (EndPoint endp, char *buffer, size_t *len)
+void cancelWrite (EndPoint endp, char *buffer UNUSED, size_t *len UNUSED)
 {
-  (void) buffer ;
-  (void) len ;
-  
   FD_CLR (endp->myFd, &wrSet) ;
   if (!FD_ISSET (endp->myFd, &rdSet))
     FD_CLR (endp->myFd, &exSet) ;
@@ -650,16 +586,17 @@ void Run (void)
   fd_set rSet ;
   fd_set wSet ;
   fd_set eSet ;
+  unsigned long last_summary = 0 ;
 
   keepSelecting = 1 ;
-  signal (SIGPIPE, pipeHandler) ;
+  xsignal (SIGPIPE, pipeHandler) ;
 
   while (keepSelecting)
     {
       struct timeval timeout ;
       struct timeval *twait ;
       int sval ;
-      u_int idx ;
+      unsigned int idx ;
       bool modifiedTime = false ;
       
       twait = getTimeout (&timeout) ;
@@ -687,11 +624,25 @@ void Run (void)
         }
 
       /* calculate host backlog statistics */
+      TMRstart(TMR_BACKLOGSTATS);
       gCalcHostBlStat ();
+      TMRstop(TMR_BACKLOGSTATS);
 
+      TMRstart(TMR_IDLE);
       sval = select (highestFd + 1, &rSet, &wSet, &eSet, twait) ;
+      TMRstop(TMR_IDLE);
 
       timePasses () ;
+      if (innconf->timer)
+        {
+	  unsigned long now = TMRnow () ;
+	  if (last_summary == 0 
+	      || (long) (now - last_summary) > (innconf->timer * 1000))
+	    {
+	      TMRsummary ("ME", timer_name) ;
+	      last_summary = now;
+	    }
+	}
       
       if (sval == 0 && twait == NULL)
         die ("No fd's ready and no timeouts") ;
@@ -701,7 +652,7 @@ void Run (void)
         }
       else if (sval < 0) 
         {
-          syslog (LOG_ERR,BAD_SELECT,sval) ;
+          syswarn ("ME exception: select failed: %d", sval) ;
           stopRun () ;
         }
       else if (sval > 0)
@@ -770,7 +721,8 @@ void Run (void)
                             }
                           else if (sval < 0)
                             {
-                              syslog (LOG_ERR,BAD_SELECT,sval) ;
+                              syswarn ("ME exception: select failed: %d",
+                                       sval) ;
                               stopRun () ;
                               return ;
                             }
@@ -898,7 +850,9 @@ void Run (void)
 
                     ep->workCbk = NULL ;
                     ep->workData = NULL ;
+                    TMRstart(TMR_CALLBACK);
                     func (ep,data) ;
+                    TMRstop(TMR_CALLBACK);
                   }
               
             }
@@ -983,19 +937,19 @@ void freeTimeoutQueue (void)
 static IoStatus doRead (EndPoint endp) 
 {
   int i = 0 ;
-  u_int idx ;
-  u_int bCount = 0 ;
+  unsigned int idx ;
+  unsigned int bCount = 0 ;
   struct iovec *vp = NULL ;
   Buffer *buffers = endp->inBuffer ;
-  u_int currIdx = endp->inBufferIdx ;
+  unsigned int currIdx = endp->inBufferIdx ;
   size_t amt = 0 ;
   IoStatus rval = IoIncomplete ;
 
+  TMRstart(TMR_READ);
   for (i = currIdx ; buffers && buffers [i] != NULL ; i++)
     bCount++ ;
 
-  /* if UIO_MAXIOV is undefined then set MAX_WRITEV_VEC in config.h */
-  bCount = (bCount > UIO_MAXIOV ? UIO_MAXIOV : bCount) ;
+  bCount = (bCount > IOV_MAX ? IOV_MAX : bCount) ;
 
   i = 0 ;
 
@@ -1005,8 +959,7 @@ static IoStatus doRead (EndPoint endp)
       char *bbase ;
       size_t bds, bs ;
 
-      vp = CALLOC (struct iovec, bCount) ;
-      ASSERT (vp != NULL) ;
+      vp = xcalloc (bCount, sizeof(struct iovec)) ;
 
       bbase = bufferBase (buffers[currIdx]) ;
       bds = bufferDataSize (buffers[currIdx]) ;
@@ -1039,7 +992,7 @@ static IoStatus doRead (EndPoint endp)
           endp->inAmtRead += readAmt ;
           
           /* check if we filled the first buffer */
-          if (readAmt >= vp[0].iov_len)
+          if (readAmt >= (size_t) vp[0].iov_len)
             {                   /* we did */
               bufferIncrDataSize (buffers[currIdx], vp[0].iov_len) ;
               readAmt -= vp [0].iov_len ;
@@ -1096,39 +1049,37 @@ static IoStatus doRead (EndPoint endp)
       else                   /* i < 0 && errno == EAGAIN */
         rval = IoIncomplete ;
       
-      FREE (vp) ;
+      free (vp) ;
     }
   else
     rval = IoDone ;
-  
+  TMRstop(TMR_READ);
   return rval ;
 }
 
 /* called when the file descriptor on the endpoint is write ready. */
 static IoStatus doWrite (EndPoint endp)
 {
-  u_int idx ;
+  unsigned int idx ;
   int i = 0 ;
   size_t amt = 0 ;
-  u_int bCount = 0 ;
+  unsigned int bCount = 0 ;
   struct iovec *vp = NULL ;
   Buffer *buffers = endp->outBuffer ;
-  u_int currIdx = endp->outBufferIdx ;
+  unsigned int currIdx = endp->outBufferIdx ;
   IoStatus rval = IoIncomplete ;
   
+  TMRstart(TMR_WRITE);
   for (i = currIdx ; buffers && buffers [i] != NULL ; i++)
     bCount++ ;
 
-  /* if UIO_MAXIOV is undefined then set MAX_WRITEV_VEC in config.h */
-  bCount = (bCount > UIO_MAXIOV ? UIO_MAXIOV : bCount) ;
+  bCount = (bCount > IOV_MAX ? IOV_MAX : bCount) ;
 
   i = 0 ;
   
   if (bCount > 0)
     {
-      vp = CALLOC (struct iovec, bCount) ;
-      ASSERT (vp != NULL) ;
-
+      vp = xcalloc (bCount, sizeof(struct iovec)) ;
 
       vp[0].iov_base = bufferBase (buffers [currIdx]) ;
       vp[0].iov_base = (char *) vp[0].iov_base + endp->outIndex ;
@@ -1169,7 +1120,7 @@ static IoStatus doWrite (EndPoint endp)
           /* now figure out which buffers got completely written */
           for (idx = 0 ; writeAmt > 0 ; idx++)
             {
-              if (writeAmt >= vp [idx].iov_len)
+              if (writeAmt >= (size_t) vp[idx].iov_len)
                 {
                   endp->outBufferIdx++ ;
                   endp->outIndex = 0 ;
@@ -1204,11 +1155,12 @@ static IoStatus doWrite (EndPoint endp)
       else
         d_printf (1,"Wrote 0 bytes in doWrite()?\n") ;
 
-      FREE (vp) ;
+      free (vp) ;
     }
   else
     rval = IoDone ;
 
+  TMRstop(TMR_WRITE);
   return rval ;
 }
 
@@ -1216,19 +1168,19 @@ static IoStatus doWrite (EndPoint endp)
 static IoStatus doExcept (EndPoint endp)
 {
   int optval;
-  ARGTYPE size ;
+  socklen_t size ;
   int fd = endPointFd (endp) ;
 
   if (getsockopt (fd, SOL_SOCKET, SO_ERROR,
-                  (GETSOCKOPT_ARG) &optval, &size) != 0)
-    syslog (LOG_ERR,GETSOCKOPT_FAILURE,fd) ;
+                  (char *) &optval, &size) != 0)
+    syswarn ("ME exception: getsockopt (%d)", fd) ;
   else if (optval != 0)
     {
       errno = optval ;
-      syslog (LOG_ERR,EXCEPTION_NOTICE,fd) ;
+      syswarn ("ME exception: fd %d", fd) ;
     }
   else
-    syslog (LOG_ERR,UNKNOWN_EXCEPTION,fd) ;
+    syswarn ("ME exception: fd %d: Unknown error", fd) ;
 
 #if 0
   sleep (5) ;
@@ -1249,15 +1201,15 @@ static void endPointPrint (EndPoint ep, FILE *fp)
 static void signalHandler (int s)
 {
   sigFlags[s] = 1 ;
-#if !defined(USE_SIGACTION) && !defined(USE_SIGVEC) && !defined(USE_SIGSET)
-  signal (s, signalHandler) ;
+#ifndef HAVE_SIGACTION
+  xsignal (s, signalHandler) ;
 #endif
 }
 
 
 static void pipeHandler (int s)
 {
-  signal (s, pipeHandler) ;
+  xsignal (s, pipeHandler) ;
 }
 
 
@@ -1265,8 +1217,8 @@ static void pipeHandler (int s)
    endpoints on their relative activity */
 static int hitCompare (const void *v1, const void *v2)
 {
-  const EndPoint e1 = *((const EndPoint *) v1) ;
-  const EndPoint e2 = *((const EndPoint *) v2) ;
+  const struct endpoint_s *e1 = *((const struct endpoint_s * const *) v1) ;
+  const struct endpoint_s *e2 = *((const struct endpoint_s * const *) v2) ;
   double e1Hit = e1->selectHits ;
   double e2Hit = e2->selectHits ;
 
@@ -1289,7 +1241,7 @@ static int hitCompare (const void *v1, const void *v2)
    active endpoints. */
 static void reorderPriorityList (void)
 {
-  int i, j ;
+  unsigned int i, j ;
   static int thisTime = 4;
 
   /* only sort every 4th time since it's so expensive */
@@ -1324,13 +1276,12 @@ static TimerElem newTimerElem (TimeoutId i, time_t w, EndpTCB f, void *d)
 
   if (timeoutPool == NULL)
     {
-      int i ;
+      unsigned int j ;
 
-      timeoutPool = ALLOC (TimerElemStruct, TIMEOUT_POOL_SIZE) ;
-      ASSERT (timeoutPool != NULL) ;
+      timeoutPool = xmalloc (sizeof(TimerElemStruct) * TIMEOUT_POOL_SIZE) ;
 
-      for (i = 0; i < TIMEOUT_POOL_SIZE - 1; i++)
-        timeoutPool[i] . next = &(timeoutPool [i + 1]) ;
+      for (j = 0; j < TIMEOUT_POOL_SIZE - 1; j++)
+        timeoutPool[j] . next = &(timeoutPool [j + 1]) ;
       timeoutPool [TIMEOUT_POOL_SIZE-1] . next = NULL ;
     }
 
@@ -1447,11 +1398,6 @@ static void doTimeout (void)
 #if defined (WANT_MAIN)
 
 
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/socket.h>
-#include <string.h>
-
 #define BUFF_SIZE 100
 
 
@@ -1641,7 +1587,7 @@ int main (int argc, char **argv)
   EndPoint accConn ;
   struct sockaddr_in accNet ;
   int accFd = socket (AF_INET,SOCK_STREAM,0) ;
-  u_short port = atoi (argc > 1 ? argv[1] : "10000") ;
+  unsigned short port = atoi (argc > 1 ? argv[1] : "10000") ;
   time_t t = theTime() ;
 
 
@@ -1659,7 +1605,11 @@ int main (int argc, char **argv)
   accNet.sin_addr.s_addr = htonl (INADDR_ANY) ;
   accNet.sin_port = htons (port) ;
 
+#ifdef LOG_PERROR
   openlog (program, LOG_PERROR | LOG_PID, LOG_NEWS) ;
+#else
+  openlog (program, LOG_PID, LOG_NEWS) ;
+#endif
   
   if (bind (accFd, (struct sockaddr *) &accNet, sizeof (accNet)) < 0)
     {
@@ -1689,12 +1639,12 @@ int main (int argc, char **argv)
 /* Probably doesn't do the right thing for SIGCHLD */
 void setSigHandler (int signum, void (*ptr)(int))
 {
-  u_int i ;
+  unsigned int i ;
 
   if (sigHandlers == NULL)
     {
-      sigHandlers = ALLOC (sigfn,NSIG) ;
-      sigFlags = ALLOC (int,NSIG) ;
+      sigHandlers = xmalloc (sizeof(sigfn) * NSIG) ;
+      sigFlags = xmalloc (sizeof(int) * NSIG) ;
       for (i = 0 ; i < NSIG ; i++)
         {
           sigHandlers [i] = NULL ;
@@ -1709,35 +1659,9 @@ void setSigHandler (int signum, void (*ptr)(int))
       return ;
     }
 
-#if defined(USE_SIGACTION)
-  {
-    struct sigaction sa ;
-
-    sa.sa_handler = signalHandler ;
-    sa.sa_flags = 0 ;
-    sigemptyset (&sa.sa_mask) ;
-
-    if (sigaction ( signum, &sa, (struct sigaction *)NULL) != 0)
-      die ("sigaction failed: %s", strerror(errno)) ;
-  }
-#elif defined(USE_SIGVEC)
-  {
-    struct sigvec vec;
-
-    vec.sv_handler = signalHandler ;
-    vec.sv_mask = 0 ;
-    vec.sv_flags = 0 ;
-
-    if (sigvec (signum, &vec, (struct sigvec *)NULL) != 0)
-      die ("sigvec failed: %s", strerror(errno)) ;
-  }
-#elif defined(USE_SIGSET)
-  if (sigset (signum, signalHandler) == SIG_ERR)
-    die ("sigset failed: %s", strerror(errno)) ;
-#else
-  if (signal (signum, signalHandler) == -1)
+  if (xsignal (signum, signalHandler) == SIG_ERR)
     die ("signal failed: %s", strerror(errno)) ;
-#endif
+
   sigHandlers[signum] = ptr ;
 }
 
@@ -1784,7 +1708,7 @@ static void handleSignals (void)
           if (sigprocmask (SIG_SETMASK, &oset, (sigset_t *)NULL) != 0)
             die ("sigprocmask failed: %s", strerror(errno)) ;
 #elif defined(USE_SIGVEC)
-          (void) sigsetmask (mask) ;
+          sigsetmask (mask) ;
 #elif defined(USE_SIGSET)
           if (sigrelse (i) != 0)
             die ("sigrelse failed: %s", strerror(errno)) ;
@@ -1819,7 +1743,10 @@ int endpointConfigLoadCbk (void *data)
 
       if (stdioFdMax > FD_SETSIZE)
         {
-          logOrPrint (LOG_ERR,fp,INT_TO_HIGH,"stdio-fdmax",ival,"global scope",
+          logOrPrint (LOG_ERR,fp,
+                      "ME config: value of %s (%ld) in %s is higher"
+                      " than maximum of %ld. Using %ld","stdio-fdmax",
+                      ival,"global scope",
                       (long) FD_SETSIZE, (long) FD_SETSIZE) ;
           stdioFdMax = FD_SETSIZE ;
           rval = 0 ;
@@ -1839,9 +1766,9 @@ int endpointConfigLoadCbk (void *data)
 #if 0
 /* definitely not the fastest, but the most portable way to find the first
   set bit in a mask  */
-static int ff_set (fd_set *set,u_int start)
+static int ff_set (fd_set *set,unsigned int start)
 {
-  u_int i ;
+  unsigned int i ;
 
   for (i = start ; i < FD_SETSIZE ; i++)
     if (FD_ISSET (i,set))
@@ -1851,9 +1778,9 @@ static int ff_set (fd_set *set,u_int start)
 }
 
 
-static int ff_free (fd_set *set, u_int start)
+static int ff_free (fd_set *set, unsigned int start)
 {
-  u_int i ;
+  unsigned int i ;
 
   for (i = start ; i < FD_SETSIZE ; i++)
     if (!FD_ISSET (i,set))
@@ -1867,7 +1794,10 @@ static int ff_free (fd_set *set, u_int start)
 
 static void endpointCleanup (void)
 {
-  FREE (endPoints) ;
-  FREE (priorityList) ;
-  FREE (sigHandlers) ;
+  free (endPoints) ;
+  free (priorityList) ;
+  free (sigHandlers) ;
+  endPoints = NULL ;
+  priorityList = NULL ;
+  sigHandlers = NULL ;
 }
