@@ -1,14 +1,15 @@
-/*  $Revision: 1.4 $
+/*  $Id: radix32.c,v 1.5 2003/01/13 06:44:23 rra Exp $
 **
 **  Radix-32 strings divide a number into five-bit nibbles and use the
 **  alphabet 0..9a..v to represent 0..32.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <time.h>
 
+#include "libinn.h"
+
 
 static char	ALPHABET[] =
     "0123456789abcdefghijklmnopqrstuv";
@@ -18,13 +19,10 @@ static char	ALPHABET[] =
 **  Turn a number into a Radix-32 string.  Assume the number fits into
 **  32 bits.
 */
-void
-Radix32(l, buff)
-    register unsigned long	l;
-    register char		*buff;
+void Radix32(unsigned long l, char *buff)
 {
-    register char		*p;
-    register int		i;
+    char		        *p;
+    int		                i;
     char			temp[10];
 
     /* Simple sanity checks. */
@@ -51,10 +49,10 @@ Radix32(l, buff)
 */
 unsigned long
 Decode32(p)
-    register char		*p;
+    char		*p;
 {
-    register unsigned long	l;
-    register char		*cp;
+    unsigned long	l;
+    char		*cp;
 
     for (l = 0; *p; p++) {
 	if ((cp = strchr(ALPHABET, *p)) == NULL)
